@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import copy
+import re
 from abc import abstractmethod
 from dataclasses import dataclass, is_dataclass
 from typing import Callable, Dict, List, Optional, Tuple, Union
@@ -363,6 +364,8 @@ class AbstractRNNTDecoding(ConfidenceMixin):
                 preserve_alignments=self.preserve_alignments,
                 ngram_lm_model=self.cfg.beam.get('ngram_lm_model', None),
                 ngram_lm_alpha=self.cfg.beam.get('ngram_lm_alpha', 0.0),
+                hat_subtract_ilm=self.cfg.beam.get('hat_subtract_ilm', False),
+                hat_ilm_weight=self.cfg.beam.get('hat_ilm_weight', 0.0),
             )
 
         else:
@@ -497,6 +500,10 @@ class AbstractRNNTDecoding(ConfidenceMixin):
             else:
                 hypothesis = self.decode_tokens_to_str(prediction)
 
+                # TODO: remove
+                # collapse leading spaces before . , ? for PC models
+                hypothesis = re.sub(r'(\s+)([\.\,\?])', r'\2', hypothesis)
+
                 if self.compute_hypothesis_token_set:
                     hypotheses_list[ind].tokens = self.decode_ids_to_tokens(prediction)
 
@@ -1168,7 +1175,7 @@ class RNNTWER(Metric):
     def __init__(
         self, decoding: RNNTDecoding, batch_dim_index=0, use_cer=False, log_prediction=True, dist_sync_on_step=False
     ):
-        super(RNNTWER, self).__init__(dist_sync_on_step=dist_sync_on_step, compute_on_step=False)
+        super(RNNTWER, self).__init__(dist_sync_on_step=dist_sync_on_step)
         self.decoding = decoding
         self.batch_dim_index = batch_dim_index
         self.use_cer = use_cer
@@ -1261,3 +1268,6 @@ class RNNTDecodingConfig:
 
     # beam decoding config
     beam: beam_decode.BeamRNNTInferConfig = beam_decode.BeamRNNTInferConfig(beam_size=4)
+
+    # can be used to change temperature for decoding
+    temperature: float = 1.0
