@@ -62,7 +62,7 @@ class RNNTBPEDecoding(AbstractRNNTDecoding):
                 The timestamps will be available in the returned Hypothesis.timestep as a dictionary.
 
             compute_langs: a bool flag, which allows to compute language id (LID) information per token,
-                word, and the entire sample (most likely language id). The LIDS will be available 
+                word, and the entire sample (most likely language id). The LIDS will be available
                 in the returned Hypothesis object as a dictionary
 
             rnnt_timestamp_type: A str value, which represents the types of timestamps that should be calculated.
@@ -354,7 +354,7 @@ class RNNTBPEWER(Metric):
         log_prediction: bool = True,
         dist_sync_on_step=False,
     ):
-        super(RNNTBPEWER, self).__init__(dist_sync_on_step=dist_sync_on_step, compute_on_step=False)
+        super(RNNTBPEWER, self).__init__(dist_sync_on_step=dist_sync_on_step)
         self.decoding = decoding
         self.batch_dim_index = batch_dim_index
         self.use_cer = use_cer
