@@ -17,12 +17,13 @@ import json
 import os
 import tempfile
 from abc import abstractmethod
-from math import ceil
+from math import ceil, floor
 from typing import Dict, List, Optional, Union
 
 import torch
 from omegaconf import DictConfig, ListConfig, OmegaConf
 from pytorch_lightning import Trainer
+from torchmetrics import Accuracy
 from torchmetrics.regression import MeanAbsoluteError, MeanSquaredError
 
 from nemo.collections.asr.data import audio_to_label_dataset, feature_to_label_dataset
@@ -199,7 +200,7 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
         # preserve config
         self._update_dataset_config(dataset_name='test', config=test_data_config)
 
-        if use_feat:
+        if use_feat and hasattr(self, '_setup_feature_label_dataloader'):
             self._test_dl = self._setup_feature_label_dataloader(config=DictConfig(test_data_config))
         else:
             self._test_dl = self._setup_dataloader_from_config(config=DictConfig(test_data_config))
@@ -250,9 +251,13 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
             shuffle = False
             batch_size = config['batch_size']
             if hasattr(dataset, 'collate_fn'):
-                collate_func = dataset.collate_fn
+                collate_fn = dataset.collate_fn
+            elif hasattr(dataset.datasets[0], 'collate_fn'):
+                # support datasets that are lists of entries
+                collate_fn = dataset.datasets[0].collate_fn
             else:
-                collate_func = dataset.datasets[0].collate_fn
+                # support datasets that are lists of lists
+                collate_fn = dataset.datasets[0].datasets[0].collate_fn
 
         else:
             if 'manifest_filepath' in config and config['manifest_filepath'] is None:
@@ -263,19 +268,23 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
                 logging.info("Perform streaming frame-level VAD")
                 dataset = audio_to_label_dataset.get_speech_label_dataset(featurizer=featurizer, config=config)
                 batch_size = 1
-                collate_func = dataset.vad_frame_seq_collate_fn
+                collate_fn = dataset.vad_frame_seq_collate_fn
             else:
                 dataset = audio_to_label_dataset.get_classification_label_dataset(featurizer=featurizer, config=config)
                 batch_size = config['batch_size']
                 if hasattr(dataset, 'collate_fn'):
-                    collate_func = dataset.collate_fn
+                    collate_fn = dataset.collate_fn
+                elif hasattr(dataset.datasets[0], 'collate_fn'):
+                    # support datasets that are lists of entries
+                    collate_fn = dataset.datasets[0].collate_fn
                 else:
-                    collate_func = dataset.datasets[0].collate_fn
+                    # support datasets that are lists of lists
+                    collate_fn = dataset.datasets[0].datasets[0].collate_fn
 
         return torch.utils.data.DataLoader(
             dataset=dataset,
             batch_size=batch_size,
-            collate_fn=collate_func,
+            collate_fn=collate_fn,
             drop_last=config.get('drop_last', False),
             shuffle=shuffle,
             num_workers=config.get('num_workers', 0),
@@ -384,6 +393,7 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
                             # replace top k value with current top k
                             self._accuracy.top_k = top_k_i
                             labels_k_i = self._accuracy.top_k_predicted_labels(logits)
+                            labels_k_i = labels_k_i.cpu()
                             labels_k.append(labels_k_i)
 
                         # convenience: if only one top_k, pop out the nested list
@@ -825,3 +835,310 @@ class EncDecRegressionModel(_EncDecBaseModel):
             cfg.num_classes = 1
 
         OmegaConf.set_struct(cfg, True)
+
+
+class EncDecFrameClassificationModel(EncDecClassificationModel):
+    @property
+    def output_types(self) -> Optional[Dict[str, NeuralType]]:
+        return {"outputs": NeuralType(('B', 'T', 'C'), LogitsType())}
+
+    def __init__(self, cfg: DictConfig, trainer: Trainer = None):
+        self.num_classes = len(cfg.labels)
+        self.eval_loop_cnt = 0
+        super().__init__(cfg=cfg, trainer=trainer)
+
+    @classmethod
+    def list_available_models(cls) -> Optional[List[PretrainedModelInfo]]:
+        results = []
+        model = PretrainedModelInfo(
+            pretrained_model_name="vad_multilingual_frame_marblenet",
+            description="For details about this model, please visit https://catalog.ngc.nvidia.com/orgs/nvidia/teams/nemo/models/vad_multilingual_frame_marblenet",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/vad_multilingual_frame_marblenet/versions/1.20.0/files/vad_multilingual_frame_marblenet.nemo",
+        )
+        results.append(model)
+        return results
+
+    def _setup_metrics(self):
+        self._accuracy = TopKClassificationAccuracy(dist_sync_on_step=True)
+        self._macro_accuracy = Accuracy(num_classes=self.num_classes, average='macro', task="multiclass")
+
+    def _setup_loss(self):
+        if "loss" in self.cfg:
+            weight = self.cfg.loss.get("weight", None)
+            if weight in [None, "none", "None"]:
+                weight = [1.0] * self.num_classes
+            logging.info(f"Using cross-entropy with weights: {weight}")
+        else:
+            weight = [1.0] * self.num_classes
+        return CrossEntropyLoss(logits_ndim=3, weight=weight)
+
+    def _setup_dataloader_from_config(self, config: DictConfig):
+        OmegaConf.set_struct(config, False)
+        config.is_regression_task = self.is_regression_task
+        OmegaConf.set_struct(config, True)
+        shuffle = config.get('shuffle', False)
+
+        if config.get('is_tarred', False):
+            if ('tarred_audio_filepaths' in config and config['tarred_audio_filepaths'] is None) or (
+                'manifest_filepath' in config and config['manifest_filepath'] is None
+            ):
+                raise ValueError(
+                    "Could not load dataset as `manifest_filepath` is None or "
+                    f"`tarred_audio_filepaths` is None. Provided cfg : {config}"
+                )
+
+            shuffle_n = config.get('shuffle_n', 4 * config['batch_size']) if shuffle else 0
+            dataset = audio_to_label_dataset.get_tarred_audio_multi_label_dataset(
+                cfg=config, shuffle_n=shuffle_n, global_rank=self.global_rank, world_size=self.world_size,
+            )
+            shuffle = False
+            if hasattr(dataset, 'collate_fn'):
+                collate_func = dataset.collate_fn
+            else:
+                collate_func = dataset.datasets[0].collate_fn
+        else:
+            if 'manifest_filepath' in config and config['manifest_filepath'] is None:
+                raise ValueError(f"Could not load dataset as `manifest_filepath` is None. Provided cfg : {config}")
+            dataset = audio_to_label_dataset.get_audio_multi_label_dataset(config)
+            collate_func = dataset.collate_fn
+
+        return torch.utils.data.DataLoader(
+            dataset=dataset,
+            batch_size=config.get("batch_size", 1),
+            collate_fn=collate_func,
+            drop_last=config.get('drop_last', False),
+            shuffle=shuffle,
+            num_workers=config.get('num_workers', 0),
+            pin_memory=config.get('pin_memory', False),
+        )
+
+    def _setup_feature_label_dataloader(self, config: DictConfig) -> torch.utils.data.DataLoader:
+        """
+        setup dataloader for VAD inference with audio features as input
+        """
+
+        OmegaConf.set_struct(config, False)
+        config.is_regression_task = self.is_regression_task
+        OmegaConf.set_struct(config, True)
+
+        if 'augmentor' in config:
+            augmentor = process_augmentations(config['augmentor'])
+        else:
+            augmentor = None
+        if 'manifest_filepath' in config and config['manifest_filepath'] is None:
+            logging.warning(f"Could not load dataset as `manifest_filepath` is None. Provided config : {config}")
+            return None
+
+        dataset = feature_to_label_dataset.get_feature_multi_label_dataset(config=config, augmentor=augmentor)
+
+        return torch.utils.data.DataLoader(
+            dataset=dataset,
+            batch_size=config.get("batch_size", 1),
+            collate_fn=dataset.collate_fn,
+            drop_last=config.get('drop_last', False),
+            shuffle=config.get('shuffle', False),
+            num_workers=config.get('num_workers', 0),
+            pin_memory=config.get('pin_memory', False),
+        )
+
+    def get_label_masks(self, labels, labels_len):
+        mask = torch.arange(labels.size(1))[None, :].to(labels.device) < labels_len[:, None]
+        return mask.to(labels.device, dtype=bool)
+
+    @typecheck()
+    def forward(
+        self, input_signal=None, input_signal_length=None, processed_signal=None, processed_signal_length=None
+    ):
+        has_input_signal = input_signal is not None and input_signal_length is not None
+        has_processed_signal = processed_signal is not None and processed_signal_length is not None
+        if (has_input_signal ^ has_processed_signal) == False:
+            raise ValueError(
+                f"{self} Arguments ``input_signal`` and ``input_signal_length`` are mutually exclusive "
+                " with ``processed_signal`` and ``processed_signal_length`` arguments."
+            )
+
+        if not has_processed_signal:
+            processed_signal, processed_signal_length = self.preprocessor(
+                input_signal=input_signal, length=input_signal_length,
+            )
+
+        # Crop or pad is always applied
+        if self.crop_or_pad is not None:
+            processed_signal, processed_signal_length = self.crop_or_pad(
+                input_signal=processed_signal, length=processed_signal_length
+            )
+        # Spec augment is not applied during evaluation/testing
+        if self.spec_augmentation is not None and self.training:
+            processed_signal = self.spec_augmentation(input_spec=processed_signal, length=processed_signal_length)
+        encoded, encoded_len = self.encoder(audio_signal=processed_signal, length=processed_signal_length)
+        logits = self.decoder(encoded.transpose(1, 2))
+        return logits
+
+    # PTL-specific methods
+    def training_step(self, batch, batch_idx):
+        audio_signal, audio_signal_len, labels, labels_len = batch
+        logits = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
+        labels, labels_len = self.reshape_labels(logits, labels, audio_signal_len, labels_len)
+        masks = self.get_label_masks(labels, labels_len)
+
+        loss_value = self.loss(logits=logits, labels=labels, loss_mask=masks)
+
+        tensorboard_logs = {
+            'train_loss': loss_value,
+            'learning_rate': self._optimizer.param_groups[0]['lr'],
+            'global_step': torch.tensor(self.trainer.global_step, dtype=torch.float32),
+        }
+
+        metric_logits, metric_labels = self.get_metric_logits_labels(logits, labels, masks)
+        self._accuracy(logits=metric_logits, labels=metric_labels)
+        topk_scores = self._accuracy.compute()
+        self._accuracy.reset()
+
+        for top_k, score in zip(self._accuracy.top_k, topk_scores):
+            tensorboard_logs[f'training_batch_accuracy_top@{top_k}'] = score
+
+        return {'loss': loss_value, 'log': tensorboard_logs}
+
+    def validation_step(self, batch, batch_idx, dataloader_idx: int = 0, tag: str = 'val'):
+        audio_signal, audio_signal_len, labels, labels_len = batch
+        logits = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
+        labels, labels_len = self.reshape_labels(logits, labels, audio_signal_len, labels_len)
+        masks = self.get_label_masks(labels, labels_len)
+
+        loss_value = self.loss(logits=logits, labels=labels, loss_mask=masks)
+
+        metric_logits, metric_labels = self.get_metric_logits_labels(logits, labels, masks)
+
+        acc = self._accuracy(logits=metric_logits, labels=metric_labels)
+        correct_counts, total_counts = self._accuracy.correct_counts_k, self._accuracy.total_counts_k
+
+        self._macro_accuracy.update(preds=metric_logits, target=metric_labels)
+        stats = self._macro_accuracy._final_state()
+
+        return {
+            f'{tag}_loss': loss_value,
+            f'{tag}_correct_counts': correct_counts,
+            f'{tag}_total_counts': total_counts,
+            f'{tag}_acc_micro': acc,
+            f'{tag}_acc_stats': stats,
+        }
+
+    def multi_validation_epoch_end(self, outputs, dataloader_idx: int = 0, tag: str = 'val'):
+        val_loss_mean = torch.stack([x[f'{tag}_loss'] for x in outputs]).mean()
+        correct_counts = torch.stack([x[f'{tag}_correct_counts'] for x in outputs]).sum(axis=0)
+        total_counts = torch.stack([x[f'{tag}_total_counts'] for x in outputs]).sum(axis=0)
+
+        self._accuracy.correct_counts_k = correct_counts
+        self._accuracy.total_counts_k = total_counts
+        topk_scores = self._accuracy.compute()
+
+        self._macro_accuracy.tp = torch.stack([x[f'{tag}_acc_stats'][0] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.fp = torch.stack([x[f'{tag}_acc_stats'][1] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.tn = torch.stack([x[f'{tag}_acc_stats'][2] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.fn = torch.stack([x[f'{tag}_acc_stats'][3] for x in outputs]).sum(axis=0)
+        macro_accuracy_score = self._macro_accuracy.compute()
+
+        self._accuracy.reset()
+        self._macro_accuracy.reset()
+
+        tensorboard_log = {
+            f'{tag}_loss': val_loss_mean,
+            f'{tag}_acc_macro': macro_accuracy_score,
+        }
+
+        for top_k, score in zip(self._accuracy.top_k, topk_scores):
+            tensorboard_log[f'{tag}_acc_micro_top@{top_k}'] = score
+
+        self.log_dict(tensorboard_log, sync_dist=True)
+        return tensorboard_log
+
+    def test_step(self, batch, batch_idx, dataloader_idx=0):
+        return self.validation_step(batch, batch_idx, dataloader_idx, tag='test')
+
+    def multi_test_epoch_end(self, outputs, dataloader_idx: int = 0):
+        return self.multi_validation_epoch_end(outputs, dataloader_idx, tag='test')
+
+    def reshape_labels(self, logits, labels, logits_len, labels_len):
+        """
+        Reshape labels to match logits shape. For example, each label is expected to cover a 40ms frame, while each frme prediction from the
+        model covers 20ms. If labels are shorter than logits, labels are repeated, otherwise labels are folded and argmax is applied to obtain 
+        the label of each frame. When lengths of labels and logits are not factors of each other, labels are truncated or padded with zeros.
+        The threshold 0.2 is used to determine whether to pad or truncate labels, where the value 0.2 is not important as in real cases the ratio
+        is very close to either ceil(ratio) or floor(ratio). We use 0.2 here for easier unit-testing.
+        Args:
+            logits: logits tensor with shape [B, T1, C]
+            labels: labels tensor with shape [B, T2]
+            logits_len: logits length tensor with shape [B]
+            labels_len: labels length tensor with shape [B]
+        Returns:
+            labels: labels tensor with shape [B, T1]
+            labels_len: labels length tensor with shape [B]
+        """
+        logits_max_len = logits.size(1)
+        labels_max_len = labels.size(1)
+        batch_size = logits.size(0)
+        if logits_max_len < labels_max_len:
+            ratio = labels_max_len // logits_max_len
+            res = labels_max_len % logits_max_len
+            if ceil(ratio) - ratio < 0.2:  # e.g., ratio is 1.99
+                # pad labels with zeros until labels_max_len is a multiple of logits_max_len
+                labels = labels.cpu().tolist()
+                if len(labels) % ceil(ratio) != 0:
+                    labels += [0] * (ceil(ratio) - len(labels) % ceil(ratio))
+                labels = torch.tensor(labels).long().to(logits.device)
+                labels = labels.view(-1, ceil(ratio)).amax(1)
+                return self.reshape_labels(logits, labels, logits_len, labels_len)
+            else:
+                # truncate additional labels until labels_max_len is a multiple of logits_max_len
+                if res > 0:
+                    labels = labels[:, :-res]
+                    mask = labels_len > (labels_max_len - res)
+                    labels_len = labels_len - mask * (labels_len - (labels_max_len - res))
+                labels = labels.view(batch_size, ratio, -1).amax(1)
+                labels_len = torch.div(labels_len, ratio, rounding_mode="floor")
+                labels_len = torch.min(torch.cat([logits_len[:, None], labels_len[:, None]], dim=1), dim=1)[0]
+                return labels.contiguous(), labels_len.contiguous()
+        elif logits_max_len > labels_max_len:
+            ratio = logits_max_len / labels_max_len
+            res = logits_max_len % labels_max_len
+            if ceil(ratio) - ratio < 0.2:  # e.g., ratio is 1.99
+                # repeat labels for ceil(ratio) times, and DROP additional labels based on logits_max_len
+                labels = labels.repeat_interleave(ceil(ratio), dim=1).long()
+                labels = labels[:, :logits_max_len]
+                labels_len = labels_len * ceil(ratio)
+                mask = labels_len > logits_max_len
+                labels_len = labels_len - mask * (labels_len - logits_max_len)
+            else:  # e.g., ratio is 2.01
+                # repeat labels for floor(ratio) times, and ADD padding labels based on logits_max_len
+                labels = labels.repeat_interleave(floor(ratio), dim=1).long()
+                labels_len = labels_len * floor(ratio)
+                if res > 0:
+                    labels = torch.cat([labels, labels[:, -res:]], dim=1)
+                    # no need to update `labels_len` since we ignore additional "res" padded labels
+            labels_len = torch.min(torch.cat([logits_len[:, None], labels_len[:, None]], dim=1), dim=1)[0]
+            return labels.contiguous(), labels_len.contiguous()
+        else:
+            labels_len = torch.min(torch.cat([logits_len[:, None], labels_len[:, None]], dim=1), dim=1)[0]
+            return labels, labels_len
+
+    def get_metric_logits_labels(self, logits, labels, masks):
+        """
+        Computes valid logits and labels for metric computation.
+        Args:
+           logits: tensor of shape [B, T, C]
+           labels: tensor of shape [B, T]
+           masks: tensor of shape [B, T]
+        Returns:
+           logits of shape [N, C]
+           labels of shape [N,]
+        """
+        C = logits.size(2)
+        logits = logits.view(-1, C)  # [BxT, C]
+        labels = labels.view(-1).contiguous()  # [BxT,]
+        masks = masks.view(-1)  # [BxT,]
+        idx = masks.nonzero()  # [BxT, 1]
+
+        logits = logits.gather(dim=0, index=idx.repeat(1, 2))
+        labels = labels.gather(dim=0, index=idx.view(-1))
+
+        return logits, labels
