@@ -15,7 +15,7 @@ import glob
 import json
 import os
 import re
-from pathlib import Path
+from dataclasses import dataclass
 from typing import List, Optional, Tuple, Union
 
 import torch
@@ -27,6 +27,7 @@ from nemo.collections.asr.models import ASRModel
 from nemo.collections.asr.models.ctc_models import EncDecCTCModel
 from nemo.collections.asr.parts.utils import rnnt_utils
 from nemo.collections.asr.parts.utils.streaming_utils import FrameBatchASR
+from nemo.collections.common.parts.preprocessing.manifest import get_full_path
 from nemo.utils import logging, model_utils
 
 
@@ -57,7 +58,8 @@ def get_buffered_pred_feat_rnnt(
             print("Parsing manifest files...")
             for l in mfst_f:
                 row = json.loads(l.strip())
-                filepaths.append(row['audio_filepath'])
+                audio_file = get_full_path(audio_file=row['audio_filepath'], manifest_file=manifest)
+                filepaths.append(audio_file)
                 if 'text' in row:
                     refs.append(row['text'])
 
@@ -148,8 +150,9 @@ def get_buffered_pred_feat(
                 row = json.loads(l.strip())
                 if 'text' in row:
                     refs.append(row['text'])
+                audio_file = get_full_path(audio_file=row['audio_filepath'], manifest_file=manifest)
                 # do not support partial audio
-                asr.read_audio_file(row['audio_filepath'], delay, model_stride_in_secs)
+                asr.read_audio_file(audio_file, delay, model_stride_in_secs)
                 hyp = asr.transcribe(tokens_per_chunk, delay)
                 hyps.append(hyp)
 
@@ -217,7 +220,6 @@ def prepare_audio_data(cfg: DictConfig) -> Tuple[List[str], bool]:
             logging.error(f"The input dataset_manifest {cfg.dataset_manifest} is empty. Exiting!")
             return None
 
-        manifest_dir = Path(cfg.dataset_manifest).parent
         with open(cfg.dataset_manifest, 'r', encoding='utf_8') as f:
             has_two_fields = []
             for line in f:
@@ -227,10 +229,8 @@ def prepare_audio_data(cfg: DictConfig) -> Tuple[List[str], bool]:
                 else:
                     has_two_fields.append(False)
                 audio_key = cfg.get('audio_key', 'audio_filepath')
-                audio_file = Path(item[audio_key])
-                if not audio_file.is_file() and not audio_file.is_absolute():
-                    audio_file = manifest_dir / audio_file
-                filepaths.append(str(audio_file.absolute()))
+                audio_file = get_full_path(audio_file=item[audio_key], manifest_file=cfg.dataset_manifest)
+                filepaths.append(audio_file)
         partial_audio = all(has_two_fields)
     logging.info(f"\nTranscribing {len(filepaths)} files...\n")
 
@@ -278,7 +278,7 @@ def write_transcription(
     filepaths: List[str] = None,
     compute_langs: bool = False,
     compute_timestamps: bool = False,
-) -> str:
+) -> Tuple[str, str]:
     """ Write generated transcription to output file. """
     if cfg.append_pred:
         logging.info(f'Transcripts will be written in "{cfg.output_filename}" file')
@@ -292,14 +292,14 @@ def write_transcription(
 
     if isinstance(transcriptions[0], rnnt_utils.Hypothesis):  # List[rnnt_utils.Hypothesis]
         best_hyps = transcriptions
-        assert cfg.ctc_decoding.beam.return_best_hypothesis, "Works only with return_best_hypothesis=true"
+        assert cfg.decoding.beam.return_best_hypothesis, "Works only with return_best_hypothesis=true"
     elif isinstance(transcriptions[0], list) and isinstance(
         transcriptions[0][0], rnnt_utils.Hypothesis
     ):  # List[List[rnnt_utils.Hypothesis]] NBestHypothesis
         best_hyps, beams = [], []
         for hyps in transcriptions:
             best_hyps.append(hyps[0])
-            if not cfg.ctc_decoding.beam.return_best_hypothesis:
+            if not cfg.decoding.beam.return_best_hypothesis:
                 beam = []
                 for hyp in hyps:
                     beam.append((hyp.text, hyp.score))
@@ -323,11 +323,11 @@ def write_transcription(
                 if compute_langs:
                     item['pred_lang'] = transcription.langs
                     item['pred_lang_chars'] = transcription.langs_chars
-                if not cfg.ctc_decoding.beam.return_best_hypothesis:
+                if not cfg.decoding.beam.return_best_hypothesis:
                     item['beams'] = beams[idx]
                 f.write(json.dumps(item) + "\n")
         else:
-            with open(cfg.dataset_manifest, 'r', encoding='utf_8') as fr:
+            with open(cfg.dataset_manifest, 'r', encoding='utf-8') as fr:
                 for idx, line in enumerate(fr):
                     item = json.loads(line)
                     item[pred_text_attr_name] = best_hyps[idx].text
@@ -346,11 +346,11 @@ def write_transcription(
                         item['pred_lang'] = best_hyps[idx].langs
                         item['pred_lang_chars'] = best_hyps[idx].langs_chars
 
-                    if not cfg.ctc_decoding.beam.return_best_hypothesis:
+                    if not cfg.decoding.beam.return_best_hypothesis:
                         item['beams'] = beams[idx]
                     f.write(json.dumps(item) + "\n")
 
-    return cfg.output_filename
+    return cfg.output_filename, pred_text_attr_name
 
 
 def transcribe_partial_audio(
@@ -362,11 +362,11 @@ def transcribe_partial_audio(
     num_workers: int = 0,
     channel_selector: Optional[int] = None,
     augmentor: DictConfig = None,
+    decoder_type: Optional[str] = None,
 ) -> List[str]:
     """
-    See description of this function in trancribe() in nemo/collections/asr/models/ctc_models.py    """
-
-    assert isinstance(asr_model, EncDecCTCModel), "Currently support CTC model only."
+    See description of this function in trancribe() in nemo/collections/asr/models/ctc_models.py and nemo/collections/asr/models/rnnt_models.py
+    """
 
     if return_hypotheses and logprobs:
         raise ValueError(
@@ -384,6 +384,17 @@ def transcribe_partial_audio(
     dither_value = asr_model.preprocessor.featurizer.dither
     pad_to_value = asr_model.preprocessor.featurizer.pad_to
 
+    if decoder_type is not None:  # Hybrid model
+        decode_function = (
+            asr_model.decoding.rnnt_decoder_predictions_tensor
+            if decoder_type == 'rnnt'
+            else asr_model.decoding.ctc_decoder_predictions_tensor
+        )
+    elif hasattr(asr_model, 'joint'):  # RNNT model
+        decode_function = asr_model.decoding.rnnt_decoder_predictions_tensor
+    else:  # CTC model
+        decode_function = asr_model.decoding.ctc_decoder_predictions_tensor
+
     try:
         asr_model.preprocessor.featurizer.dither = 0.0
         asr_model.preprocessor.featurizer.pad_to = 0
@@ -406,18 +417,20 @@ def transcribe_partial_audio(
 
         temporary_datalayer = asr_model._setup_transcribe_dataloader(config)
         for test_batch in tqdm(temporary_datalayer, desc="Transcribing"):
-            logits, logits_len, greedy_predictions = asr_model.forward(
+            outputs = asr_model.forward(
                 input_signal=test_batch[0].to(device), input_signal_length=test_batch[1].to(device)
             )
+            logits, logits_len = outputs[0], outputs[1]
             if logprobs:
                 # dump log probs per file
                 for idx in range(logits.shape[0]):
                     lg = logits[idx][: logits_len[idx]]
                     hypotheses.append(lg.cpu().numpy())
             else:
-                current_hypotheses, all_hyp = asr_model.decoding.ctc_decoder_predictions_tensor(
-                    logits, decoder_lengths=logits_len, return_hypotheses=return_hypotheses,
-                )
+                current_hypotheses, all_hyp = decode_function(logits, logits_len, return_hypotheses=return_hypotheses,)
+
+                if isinstance(current_hypotheses, tuple) and len(current_hypotheses) == 2:
+                    current_hypotheses = current_hypotheses[0]
 
                 if return_hypotheses:
                     # dump log probs per file
@@ -428,7 +441,6 @@ def transcribe_partial_audio(
 
                 hypotheses += current_hypotheses
 
-            del greedy_predictions
             del logits
             del test_batch
 
@@ -445,14 +457,48 @@ def transcribe_partial_audio(
 
 
 class PunctuationCapitalization:
-    def __init__(self, punctuation_marks='.,?'):
-        self.regex_punctuation = re.compile(fr"([{''.join(punctuation_marks)}])")
+    def __init__(self, punctuation_marks: str):
+        """
+        Class for text processing with punctuation and capitalization. Can be used with class TextProcessingConfig.
 
-    def separate_punctuation(self, lines):
-        return [self.regex_punctuation.sub(r' \1 ', line) for line in lines]
+        Args:
+            punctuation_marks (str): String with punctuation marks to process.
+        Example: punctuation_marks = '.,?'
+        """
+        if punctuation_marks:
+            self.regex_punctuation = re.compile(fr"([{''.join(punctuation_marks)}])")
+            self.regex_extra_space = re.compile('\s{2,}')
+        else:
+            self.regex_punctuation = None
 
-    def do_lowercase(self, lines):
+    def separate_punctuation(self, lines: List[str]) -> List[str]:
+        if self.regex_punctuation is not None:
+            return [
+                self.regex_extra_space.sub(' ', self.regex_punctuation.sub(r' \1 ', line)).strip() for line in lines
+            ]
+        else:
+            return lines
+
+    def do_lowercase(self, lines: List[str]) -> List[str]:
         return [line.lower() for line in lines]
 
-    def rm_punctuation(self, lines):
-        return [self.regex_punctuation.sub(' ', line).strip() for line in lines]
+    def rm_punctuation(self, lines: List[str]) -> List[str]:
+        if self.regex_punctuation is not None:
+            return [self.regex_extra_space.sub(' ', self.regex_punctuation.sub(' ', line)).strip() for line in lines]
+        else:
+            return lines
+
+
+@dataclass
+class TextProcessingConfig:
+    # Punctuation marks to process. Example: ".,?"
+    punctuation_marks: str = ""
+
+    # Whether to apply lower case conversion on the training text.
+    do_lowercase: bool = False
+
+    # Whether to remove punctuation marks from text.
+    rm_punctuation: bool = False
+
+    # Whether to separate punctuation with the previouse word by space.
+    separate_punctuation: bool = True
