@@ -13,6 +13,7 @@
 # limitations under the License.
 
 from torchmetrics import Accuracy, AveragePrecision, F1Score, MatthewsCorrCoef, PearsonCorrCoef, SpearmanCorrCoef
+from torchmetrics.text.rouge import ROUGEScore
 
 from nemo.collections.common.metrics.classification_accuracy import ExactStringMatchMetric
 
@@ -28,4 +29,5 @@ MetricStringToTorchMetric = {
     'spearman_corr_coef': SpearmanCorrCoef,
     'matthews_corr_coef': MatthewsCorrCoef,
     'exact_string_match': ExactStringMatchMetric,
+    'rouge': ROUGEScore,
 }
