@@ -31,7 +31,7 @@ __all__ = ['L2RLanguageModelingDataset', 'TarredL2RLanguageModelingDataset']
 class L2RLanguageModelingDataset(Dataset):
     """
     Dataset for training and evaluating left-to-right language models.
-    
+
     Args:
         tokenizer: tokenizer, such as WordTokenizer or CharTokenizer
         dataset: path to data
@@ -73,7 +73,7 @@ class L2RLanguageModelingDataset(Dataset):
 class TarredL2RLanguageModelingDataset(IterableDataset):
     """
     A similar Dataset to the L2RLanguageModelingDataset, but which loads tarred tokenized numpy files.
-    Accepts a single JSON metadata manifest file as well as the path(s) to the tarball(s) containing the wav files. 
+    Accepts a single JSON metadata manifest file as well as the path(s) to the tarball(s) containing the wav files.
     The manifest should contain information such as the number of shards, the number of tokens in the corpus,
     and the number of tokens contained within each shard of the tarfile(s).
 
@@ -114,10 +114,15 @@ class TarredL2RLanguageModelingDataset(IterableDataset):
                 available in the tarred dataset, which are permanently pre-allocated and never changed at runtime.
                 The benefit of replication is that it allows each node to sample data points from the entire
                 dataset independently of other nodes, and reduces dependence on value of `shuffle_n`.
-                Note: Replicated strategy allows every node to sample the entire set of available tarfiles,
-                and therefore more than one node may sample the same tarfile, and even sample the same
-                data points! As such, there is no assured guarantee that all samples in the dataset will be
-                sampled at least once during 1 epoch.
+
+                .. warning::
+                    Replicated strategy allows every node to sample the entire set of available tarfiles,
+                    and therefore more than one node may sample the same tarfile, and even sample the same
+                    data points! As such, there is no assured guarantee that all samples in the dataset will be
+                    sampled at least once during 1 epoch. Scattered strategy, on the other hand, on specific
+                    occasions (when the number of shards is not divisible with ``world_size``), will not sample
+                    the entire dataset. For these reasons it is not advisable to use tarred datasets as validation
+                    or test datasets.
         global_rank (int): Worker rank, used for partitioning shards. Defaults to 0.
         world_size (int): Total number of processes, used for partitioning shards. Defaults to 0.
     """
@@ -142,7 +147,11 @@ class TarredL2RLanguageModelingDataset(IterableDataset):
 
         valid_shard_strategies = ['scatter', 'replicate']
         if shard_strategy not in valid_shard_strategies:
-            raise ValueError(f"`shard_strategy` must be one of {valid_shard_strategies}")
+            raise ValueError(
+                f"Invalid shard strategy of type {type(shard_strategy)} "
+                f"{repr(shard_strategy) if len(repr(shard_strategy)) < 100 else repr(shard_strategy)[:100] + '...'}! "
+                f"Allowed values are: {valid_shard_strategies}."
+            )
 
         with open(metadata_path, 'r') as f:
             metadata = json.load(f)
