@@ -32,18 +32,19 @@ from nemo.core import Dataset
 from nemo.utils import logging
 
 try:
-    from apex.transformer import parallel_state
+    from megatron.core import parallel_state
 
-    HAVE_APEX = True
+    HAVE_MEGATRON_CORE = True
 
 except (ImportError, ModuleNotFoundError):
 
-    HAVE_APEX = False
+    HAVE_MEGATRON_CORE = False
 
 
 def build_dataset(cfg, trainer, data_prefix, data_impl, num_samples, seq_length, seed, skip_warmup, tokenizer, name):
     def _build_dataset(current_data_prefix, current_num_samples):
-        indexed_dataset = get_indexed_dataset_(current_data_prefix, data_impl, skip_warmup)
+        delay_data_mmap = cfg.data.get('delay_data_mmap', False)
+        indexed_dataset = get_indexed_dataset_(current_data_prefix, data_impl, skip_warmup, delay_data_mmap)
         total_num_of_documents = indexed_dataset.sizes.shape[0]
         # Print stats about the splits.
         logging.info(' > dataset split:')
@@ -91,6 +92,19 @@ def build_train_valid_test_datasets(
     skip_warmup,
     tokenizer,
 ):
+    if data_impl in ['mock']:
+        logging.info('Initializing mock GPT dataset for train, validate, and test')
+        if len(data_prefix) != 0:
+            # Mock data will be generated instead of loading files.
+            logging.warning(f"Requested data_impl={data_impl}, so ignoring data_prefix setting: {data_prefix}")
+        if tokenizer is None:
+            # Vocabulary size is inferred from tokenizer.
+            raise ValueError("Tokenizer is required for a mock GPT dataset")
+        train_ds = MockGPTDataset(cfg, tokenizer, "train", int(train_valid_test_num_samples[0]), seq_length, seed,)
+        valid_ds = MockGPTDataset(cfg, tokenizer, "valid", int(train_valid_test_num_samples[1]), seq_length, seed,)
+        test_ds = MockGPTDataset(cfg, tokenizer, "test", int(train_valid_test_num_samples[2]), seq_length, seed,)
+        return train_ds, valid_ds, test_ds
+
     if isinstance(data_prefix, DictConfig):
         assert (
             data_prefix.get('train') is not None
@@ -213,7 +227,8 @@ def _build_train_valid_test_datasets(
     """Build train, valid, and test datasets."""
 
     # Indexed dataset.
-    indexed_dataset = get_indexed_dataset_(data_prefix, data_impl, skip_warmup)
+    delay_data_mmap = cfg.data.get('delay_data_mmap', False)
+    indexed_dataset = get_indexed_dataset_(data_prefix, data_impl, skip_warmup, delay_data_mmap)
 
     total_num_of_documents = indexed_dataset.sizes.shape[0]
     splits = get_train_valid_test_split_(splits_string, total_num_of_documents)
@@ -261,12 +276,12 @@ def _build_train_valid_test_datasets(
     return (train_dataset, valid_dataset, test_dataset)
 
 
-def get_indexed_dataset_(data_prefix, data_impl, skip_warmup):
+def get_indexed_dataset_(data_prefix, data_impl, skip_warmup, delay_data_mmap=False):
     """Build indexed dataset."""
     logging.info(' > building dataset index ...')
 
     start_time = time.time()
-    indexed_dataset = make_indexed_dataset(data_prefix, data_impl, skip_warmup)
+    indexed_dataset = make_indexed_dataset(data_prefix, data_impl, skip_warmup, delay_data_mmap=delay_data_mmap)
     logging.info(' > finished creating indexed dataset in {:4f} ' 'seconds'.format(time.time() - start_time))
     logging.info('    number of documents: {}'.format(indexed_dataset.sizes.shape[0]))
 
@@ -288,9 +303,9 @@ class GPTDataset(Dataset):
         seed,
         drop_last=True,
     ):
-        if not HAVE_APEX:
+        if not HAVE_MEGATRON_CORE:
             raise ImportError(
-                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+                "megatron-core was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
             )
 
         super().__init__()
@@ -306,11 +321,15 @@ class GPTDataset(Dataset):
         self.reset_position_ids = cfg.data.get('reset_position_ids', False)
         self.reset_attention_mask = cfg.data.get('reset_attention_mask', False)
         self.eod_mask_loss = cfg.data.get('eod_mask_loss', False)
+        self.create_inputs = any([self.reset_position_ids, self.reset_attention_mask, self.eod_mask_loss])
+        self.cached_inputs = False
         self.eos_id = tokenizer.eos_id
         self.no_seqlen_plus_one_input_tokens = cfg.data.get('no_seqlen_plus_one_input_tokens', False)
         self.add_extra_token = 1
         if self.no_seqlen_plus_one_input_tokens:
             self.add_extra_token = 0
+        self.shuffle_documents = cfg.data.get('shuffle_documents', True)
+        self.exchange_indices_distributed = cfg.data.get('exchange_indices_distributed', False)
 
         # save index mappings to a configurable dir
         self.index_mapping_dir = cfg.data.get('index_mapping_dir', None)
@@ -334,9 +353,14 @@ class GPTDataset(Dataset):
             index_mapping_dir=self.index_mapping_dir,
             drop_last=drop_last,
             add_extra_token=self.add_extra_token,
+            shuffle_documents=self.shuffle_documents,
+            exchange_indices_distributed=self.exchange_indices_distributed,
         )
         deallocate_indexed_dataset_memory(self.indexed_dataset)
 
+    def create_data_mmap(self):
+        self.indexed_dataset.create_data_mmap()
+
     def __len__(self):
         # -1 is due to data structure used to retieve the index:
         #    sample i --> [sample_idx[i], sample_idx[i+1])
@@ -386,17 +410,27 @@ class GPTDataset(Dataset):
             tokens = text
             labels = torch.roll(text, shifts=-1, dims=0)
             labels[-1] = -1
-        attention_mask, loss_mask, position_ids = _create_ltor_masks_and_position_ids(
-            tokens, self.eos_id, self.reset_position_ids, self.reset_attention_mask, self.eod_mask_loss,
-        )
+        if self.create_inputs or not self.cached_inputs:
+            attention_mask, loss_mask, position_ids = _create_ltor_masks_and_position_ids(
+                tokens, self.eos_id, self.reset_position_ids, self.reset_attention_mask, self.eod_mask_loss,
+            )
+            if not self.create_inputs:
+                self.cached_attention_mask = attention_mask
+                self.cached_loss_mask = loss_mask
+                self.cached_position_ids = position_ids
+                self.cached_inputs = True
+        else:
+            attention_mask = self.cached_attention_mask
+            loss_mask = self.cached_loss_mask
+            position_ids = self.cached_position_ids
         loss_mask[labels == -1] = 0.0
         tokens[tokens == -1] = 0
         labels[labels == -1] = 0
 
         # Negative index comes when we pad the last batch in MegatronPretrainingBatchSampler
         # We make the loss_mask zero to mask out loss from these samples
-        if idx == -1:
-            logging.debug('Got -1 as item index. Masking loss from this sample')
+        if idx < 0:
+            logging.debug('Got negative index. Masking loss from this sample')
             loss_mask = torch.zeros_like(loss_mask)
 
         return {
@@ -408,6 +442,49 @@ class GPTDataset(Dataset):
         }
 
 
+class MockGPTDataset(Dataset):
+    def __init__(
+        self, cfg, tokenizer, name, num_samples, seq_length, seed,
+    ):
+        if not HAVE_MEGATRON_CORE:
+            raise ImportError(
+                "Megatron core was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+            )
+
+        super().__init__()
+        self.name = name
+        self.seq_length = seq_length
+        self.vocab_size = tokenizer.vocab_size
+        self.length = num_samples
+        self.seed = seed
+
+        self.attention_mask = torch.tril(torch.ones((self.seq_length, self.seq_length))).unsqueeze(0)
+        self.attention_mask = self.attention_mask < 0.5
+        self.loss_mask = torch.ones(self.seq_length, dtype=torch.float)
+        self.position_ids = torch.arange(self.seq_length, dtype=torch.int64)
+
+    def __len__(self):
+        return self.length
+
+    def _get_text(self, idx: int) -> np.ndarray:
+        np_gen = np.random.default_rng(seed=(self.seed + idx))
+        return np_gen.integers(self.vocab_size, size=[self.seq_length], dtype=np.int64)
+
+    def __getitem__(self, idx):
+        # Generate data of the expected size and datatype (based on GPTDataset).
+        np_gen = np.random.default_rng(seed=(self.seed + idx))
+        tokens = torch.from_numpy(np_gen.integers(self.vocab_size, size=[self.seq_length], dtype=np.int64))
+        labels = torch.from_numpy(np_gen.integers(self.vocab_size, size=[self.seq_length], dtype=np.int64))
+
+        return {
+            'tokens': tokens,
+            'labels': labels,
+            'attention_mask': self.attention_mask,
+            'loss_mask': self.loss_mask,
+            'position_ids': self.position_ids,
+        }
+
+
 @torch.no_grad()
 def _create_ltor_masks_and_position_ids(
     tokens: torch.Tensor, eod_token: int, reset_position_ids: bool, reset_attention_mask: bool, eod_mask_loss: bool,
@@ -468,6 +545,8 @@ def _build_index_mappings(
     index_mapping_dir: str = None,
     drop_last: bool = True,
     add_extra_token: int = 1,
+    shuffle_documents: bool = True,
+    exchange_indices_distributed: bool = False,
 ):
     """Build doc-idx, sample-idx, and shuffle-idx.
     doc-idx: is an array (ordered) of documents to be used in training.
@@ -496,12 +575,13 @@ def _build_index_mappings(
 
     # Build the indexed mapping if not exist.
     if torch.distributed.get_rank() == 0:
+        using_cached_indices = True
         if (
             (not os.path.isfile(doc_idx_filename))
             or (not os.path.isfile(sample_idx_filename))
             or (not os.path.isfile(shuffle_idx_filename))
         ):
-
+            using_cached_indices = False
             logging.info(' > WARNING: could not find index map files, building ' 'the indices on rank 0 ...')
 
             # For the last epoch, decide whether include the entire epoch
@@ -521,7 +601,7 @@ def _build_index_mappings(
                 last_epoch_num_samples = num_samples - num_samples_from_epochs_minus_one
                 assert last_epoch_num_samples >= 0, 'last epoch number of samples should be non-negative.'
                 num_samples_per_epoch = (tokens_per_epoch - add_extra_token) // seq_length
-                assert last_epoch_num_samples < (
+                assert last_epoch_num_samples <= (
                     num_samples_per_epoch + 1
                 ), 'last epoch number of samples exceeded max value.'
                 # If we have less than 80% of the samples for the last epoch,
@@ -545,7 +625,7 @@ def _build_index_mappings(
 
             # doc-idx.
             start_time = time.time()
-            doc_idx = _build_doc_idx(documents, num_epochs, np_rng, separate_last_epoch)
+            doc_idx = _build_doc_idx(documents, num_epochs, np_rng, separate_last_epoch, shuffle_documents)
             np.save(doc_idx_filename, doc_idx, allow_pickle=True)
             logging.info(
                 ' > elasped time to build and save doc-idx mapping '
@@ -601,17 +681,26 @@ def _build_index_mappings(
         // torch.distributed.get_world_size(group=parallel_state.get_tensor_model_parallel_group())
     )
 
-    # Load mappings.
-    start_time = time.time()
-    logging.info(' > loading doc-idx mapping from {}'.format(doc_idx_filename))
-    doc_idx = np.load(doc_idx_filename, allow_pickle=True, mmap_mode='r')
-    logging.info(' > loading sample-idx mapping from {}'.format(sample_idx_filename))
-    sample_idx = np.load(sample_idx_filename, allow_pickle=True, mmap_mode='r')
-    logging.info(' > loading shuffle-idx mapping from {}'.format(shuffle_idx_filename))
-    shuffle_idx = np.load(shuffle_idx_filename, allow_pickle=True, mmap_mode='r')
-    logging.info('    loaded indexed file in {:3.3f} seconds'.format(time.time() - start_time))
-    logging.info('    total number of samples: {}'.format(sample_idx.shape[0]))
-    logging.info('    total number of epochs: {}'.format(num_epochs))
+    if not exchange_indices_distributed or (torch.distributed.get_rank() == 0 and using_cached_indices):
+        # Load mappings.
+        start_time = time.time()
+        logging.info(' > loading doc-idx mapping from {}'.format(doc_idx_filename))
+        doc_idx = np.load(doc_idx_filename, allow_pickle=True, mmap_mode='r')
+        logging.info(' > loading sample-idx mapping from {}'.format(sample_idx_filename))
+        sample_idx = np.load(sample_idx_filename, allow_pickle=True, mmap_mode='r')
+        logging.info(' > loading shuffle-idx mapping from {}'.format(shuffle_idx_filename))
+        shuffle_idx = np.load(shuffle_idx_filename, allow_pickle=True, mmap_mode='r')
+        logging.info('    loaded indexed file in {:3.3f} seconds'.format(time.time() - start_time))
+        logging.info('    total number of samples: {}'.format(sample_idx.shape[0]))
+        logging.info('    total number of epochs: {}'.format(num_epochs))
+
+    if exchange_indices_distributed:
+        if torch.distributed.get_rank() == 0:
+            indices = [(doc_idx, sample_idx, shuffle_idx)]
+        else:
+            indices = [None]
+        torch.distributed.broadcast_object_list(indices)
+        doc_idx, sample_idx, shuffle_idx = indices[0]
 
     return doc_idx, sample_idx, shuffle_idx
 
@@ -636,7 +725,7 @@ def _num_epochs(tokens_per_epoch, seq_length, num_samples, add_extra_token=1):
             return num_epochs
 
 
-def _build_doc_idx(documents, num_epochs, np_rng, separate_last_epoch):
+def _build_doc_idx(documents, num_epochs, np_rng, separate_last_epoch, shuffle=True):
     """Build an array with length = number-of-epochs * number-of-dcuments.
     Each index is mapped to a corresponding document."""
     if not separate_last_epoch or num_epochs == 1:
@@ -644,11 +733,14 @@ def _build_doc_idx(documents, num_epochs, np_rng, separate_last_epoch):
         doc_idx[:] = documents
         doc_idx = doc_idx.reshape(-1)
         doc_idx = doc_idx.astype(np.int32)
-        np_rng.shuffle(doc_idx)
+        if shuffle:
+            np_rng.shuffle(doc_idx)
+        else:
+            logging.info('Document shuffling disabled')
         return doc_idx
 
-    doc_idx_first = _build_doc_idx(documents, num_epochs - 1, np_rng, False)
-    doc_idx_last = _build_doc_idx(documents, 1, np_rng, False)
+    doc_idx_first = _build_doc_idx(documents, num_epochs - 1, np_rng, False, shuffle)
+    doc_idx_last = _build_doc_idx(documents, 1, np_rng, False, shuffle)
     return np.concatenate((doc_idx_first, doc_idx_last))
 
 
