@@ -11,10 +11,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 from nemo.collections.nlp.models.token_classification.punctuation_capitalization_config import (
     PunctuationCapitalizationModelConfig,
 )
+from nemo.collections.nlp.models.token_classification.punctuation_capitalization_lexical_audio_model import (
+    PunctuationCapitalizationLexicalAudioModel,
+)
 from nemo.collections.nlp.models.token_classification.punctuation_capitalization_model import (
     PunctuationCapitalizationModel,
 )
