@@ -53,6 +53,7 @@ def get_decoder_model(
     post_process=True,
     init_method_std=0.02,
     use_cpu_initialization=False,
+    megatron_amp_O2=False,
     hidden_dropout=0.1,
     attention_dropout=0.1,
     ffn_dropout=0.0,
@@ -117,6 +118,7 @@ def get_decoder_model(
             pre_process=pre_process,
             post_process=post_process,
             use_cpu_initialization=use_cpu_initialization,
+            megatron_amp_O2=megatron_amp_O2,
             hidden_dropout=hidden_dropout,
             attention_dropout=attention_dropout,
             ffn_dropout=ffn_dropout,
@@ -158,6 +160,7 @@ def get_decoder_model(
             pre_process=pre_process,
             post_process=post_process,
             use_cpu_initialization=use_cpu_initialization,
+            megatron_amp_O2=megatron_amp_O2,
             hidden_dropout=hidden_dropout,
             attention_dropout=attention_dropout,
             precision=precision,
