@@ -20,10 +20,18 @@ import torch
 from nemo.utils import AppState, logging
 
 try:
-    from apex.transformer import tensor_parallel
     from apex.transformer.log_util import set_logging_level
     from apex.transformer.microbatches import ConstantNumMicroBatches
-    from apex.transformer.parallel_state import (
+    from apex.transformer.pipeline_parallel.utils import setup_microbatch_calculator
+
+    HAVE_APEX = True
+except (ImportError, ModuleNotFoundError):
+
+    HAVE_APEX = False
+
+try:
+    from megatron.core import tensor_parallel
+    from megatron.core.parallel_state import (
         get_pipeline_model_parallel_rank,
         set_pipeline_model_parallel_rank,
         set_pipeline_model_parallel_split_rank,
@@ -32,14 +40,12 @@ try:
         set_tensor_model_parallel_world_size,
         set_virtual_pipeline_model_parallel_rank,
     )
-    from apex.transformer.pipeline_parallel.utils import setup_microbatch_calculator
 
-    HAVE_APEX = True
 except (ImportError, ModuleNotFoundError):
-    HAVE_APEX = False
+
+    HAVE_MEGATRON_CORE = False
 
 try:
-    # TODO: remove when apex is updated
     from apex.transformer.parallel_state import set_virtual_pipeline_model_parallel_world_size
 
     HAVE_INTERLEAVED = True
@@ -59,13 +65,15 @@ def initialize_model_parallel_for_nemo(
     pipeline_model_parallel_split_rank=None,
     micro_batch_size=None,
     global_batch_size=None,
+    rampup_batch_size=None,
     use_fp8=False,
+    init_mpi_proc_group=False,
     seed=1234,
     apex_transformer_log_level=30,
 ):
 
     if virtual_pipeline_model_parallel_size is not None and not HAVE_INTERLEAVED:
-        raise ValueError("set_virtual_pipeline_model_parallel_world_size is needed in Apex for interleaved.")
+        raise ValueError("set_virtual_pipeline_model_parallel_world_size is needed in megatron-core for interleaved.")
 
     # updating NeMo globals
     app_state = AppState()
@@ -76,6 +84,7 @@ def initialize_model_parallel_for_nemo(
     app_state.pipeline_model_parallel_size = pipeline_model_parallel_size
     app_state.virtual_pipeline_model_parallel_size = virtual_pipeline_model_parallel_size
     app_state.use_fp8 = use_fp8
+    app_state.init_mpi_proc_group = init_mpi_proc_group
     (
         app_state.tensor_model_parallel_rank,
         app_state.pipeline_model_parallel_rank,
@@ -115,7 +124,7 @@ def initialize_model_parallel_for_nemo(
                 global_batch_size=global_batch_size,
                 micro_batch_size=micro_batch_size,
                 data_parallel_size=app_state.data_parallel_size,
-                rampup_batch_size=None,
+                rampup_batch_size=rampup_batch_size,
             )
         else:
             if isinstance(_GLOBAL_NUM_MICROBATCHES_CALCULATOR, ConstantNumMicroBatches):
@@ -171,7 +180,7 @@ def fake_initialize_model_parallel(
     """
     Fake initialize model data parallel groups so that we can instantiate model parallel models before DDP is initialized.
     This is needed because PTL execution flow is init model, init trainer -> call trainer.fit(model). DDP is initialized during .fit.
-    This function is taken from apex.transformer.parallel_state and modified so that the distributed groups are not created.
+    This function is taken from megatron.core.parallel_state and modified so that the distributed groups are not created.
     We only need the tensor parallel and pipeline parallel ranks to instantiate the model.
 
     Arguments:
