@@ -56,6 +56,7 @@ class MegatronPerceiverEncoderModule(MegatronModule):
         pre_process=True,
         post_process=True,
         use_cpu_initialization=False,
+        megatron_amp_O2=False,
         encoder_attn_mask_type=AttnMaskType.padding,
         hidden_dropout=0.1,
         attention_dropout=0.1,
@@ -124,6 +125,7 @@ class MegatronPerceiverEncoderModule(MegatronModule):
         self.ffn_dropout = ffn_dropout
         self.normalize_attention_scores = normalize_attention_scores
         self.megatron_legacy = megatron_legacy
+        self.megatron_amp_O2 = megatron_amp_O2
 
         assert self.num_self_attention_per_cross_attention >= 1
         assert self.hidden_steps >= 1
@@ -165,6 +167,7 @@ class MegatronPerceiverEncoderModule(MegatronModule):
             attention_dropout=self.attention_dropout,
             ffn_dropout=self.ffn_dropout,
             use_cpu_initialization=self.use_cpu_initialization,
+            megatron_amp_O2=self.megatron_amp_O2,
             bias_activation_fusion=self.bias_activation_fusion,
             bias_dropout_add_fusion=self.bias_dropout_add_fusion,
             masked_softmax_fusion=self.masked_softmax_fusion,
@@ -204,6 +207,7 @@ class MegatronPerceiverEncoderModule(MegatronModule):
             attention_dropout=self.attention_dropout,
             ffn_dropout=self.ffn_dropout,
             use_cpu_initialization=self.use_cpu_initialization,
+            megatron_amp_O2=self.megatron_amp_O2,
             bias_activation_fusion=self.bias_activation_fusion,
             bias_dropout_add_fusion=self.bias_dropout_add_fusion,
             masked_softmax_fusion=self.masked_softmax_fusion,
