@@ -18,6 +18,9 @@ from omegaconf import DictConfig
 from nemo.collections.nlp.modules.common.megatron.alibi_relative_position_embedding import (
     ALiBiRelativePositionEmbedding,
 )
+from nemo.collections.nlp.modules.common.megatron.kerple_relative_position_embedding import (
+    KERPLERelativePositionEmbedding,
+)
 from nemo.collections.nlp.modules.common.megatron.language_model import Embedding
 from nemo.collections.nlp.modules.common.megatron.layer_type import LayerType
 from nemo.collections.nlp.modules.common.megatron.megatron_decoders import get_decoder_model
@@ -35,18 +38,30 @@ from nemo.collections.nlp.modules.common.megatron.utils import (
     scaled_init_method_normal,
 )
 from nemo.collections.nlp.modules.common.megatron.vocab_parallel_cross_entropy import vocab_parallel_cross_entropy
+from nemo.collections.nlp.parts import utils_funcs
 
 try:
-    from apex.transformer import parallel_state, tensor_parallel
     from apex.transformer.enums import AttnMaskType, ModelType
 
     HAVE_APEX = True
+
 except (ImportError, ModuleNotFoundError):
-    HAVE_APEX = False
+
     # fake missing classes with None attributes
     AttnMaskType = ApexGuardDefaults()
     ModelType = ApexGuardDefaults()
 
+    HAVE_APEX = False
+
+try:
+    from megatron.core import parallel_state, tensor_parallel
+
+    HAVE_MEGATRON_CORE = True
+
+except (ImportError, ModuleNotFoundError):
+
+    HAVE_MEGATRON_CORE = False
+
 __all__ = ["MegatronTokenLevelHead", "MegatronTokenLevelEncoderDecoderModule"]
 
 
@@ -101,6 +116,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
         post_process=True,
         fp16_cross_entropy=False,
         use_cpu_initialization=False,
+        megatron_amp_O2=False,
         precision=16,
         embedding_init_method_std=0.02,
         embedding_dropout=0.1,
@@ -129,6 +145,8 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
 
         encoder_kv_channels, decoder_kv_channels = self._validate_config()
 
+        self.dtype = utils_funcs.dtype_from_precision(precision, megatron_amp_O2)
+
         encoder, decoder = None, None
         if add_encoder:
             if pre_process:
@@ -139,6 +157,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                     init_method=init_method_normal(embedding_init_method_std),
                     num_tokentypes=num_tokentypes,
                     use_cpu_initialization=use_cpu_initialization,
+                    dtype=self.dtype,
                     embedding_dropout_prob=embedding_dropout,
                     position_embedding_type=encoder_cfg.get('position_embedding_type', 'learned_absolute'),
                 )
@@ -165,7 +184,16 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                     num_attention_heads_alibi=None,
                     max_seq_len=max_position_embeddings,
                 )
-                self._encoder_relative_position_embedding_key = "encoder_relative_position_embedding"
+                self._encoder_relative_position_embedding_key = "encoder_alibi_position_embedding"
+            elif self.encoder_cfg.get('position_embedding_type', 'learned_absolute') == 'kerple':
+                self.encoder_relative_position_embedding = KERPLERelativePositionEmbedding(
+                    bidirectional=True,
+                    num_attention_heads=encoder_cfg.num_attention_heads,
+                    layer_type=LayerType.encoder,
+                    num_attention_heads_kerple=None,
+                    max_seq_len=max_position_embeddings,
+                )
+                self._encoder_relative_position_embedding_key = "encoder_kerple_position_embedding"
             else:
                 self.encoder_relative_position_embedding = None
 
@@ -186,6 +214,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                 post_process=post_process,
                 init_method_std=encoder_cfg.get('init_method_std', 0.02),
                 use_cpu_initialization=use_cpu_initialization,
+                megatron_amp_O2=megatron_amp_O2,
                 hidden_dropout=encoder_cfg.get('hidden_dropout', 0.1),
                 attention_dropout=encoder_cfg.get('attention_dropout', 0.1),
                 ffn_dropout=encoder_cfg.get('ffn_dropout', 0.0),
@@ -231,6 +260,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                         init_method=init_method_normal(embedding_init_method_std),
                         num_tokentypes=num_tokentypes,
                         use_cpu_initialization=use_cpu_initialization,
+                        dtype=self.dtype,
                         embedding_dropout_prob=embedding_dropout,
                         position_embedding_type=decoder_cfg.get('position_embedding_type', 'learned_absolute'),
                     )
@@ -285,7 +315,16 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                     num_attention_heads_alibi=None,
                     max_seq_len=max_position_embeddings,
                 )
-                self._decoder_relative_position_embedding_key = "decoder_relative_position_embedding"
+                self._decoder_relative_position_embedding_key = "decoder_alibi_position_embedding"
+            elif self.decoder_cfg.get('position_embedding_type', 'learned_absolute') == 'kerple':
+                self.decoder_relative_position_embedding = KERPLERelativePositionEmbedding(
+                    bidirectional=False,
+                    num_attention_heads=decoder_cfg.num_attention_heads,
+                    layer_type=LayerType.decoder,
+                    num_attention_heads_kerple=None,
+                    max_seq_len=max_position_embeddings,
+                )
+                self._decoder_relative_position_embedding_key = "decoder_kerple_position_embedding"
             else:
                 self.decoder_relative_position_embedding = None
 
@@ -306,6 +345,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                 post_process=post_process,
                 init_method_std=decoder_cfg.get('init_method_std', 0.02),
                 use_cpu_initialization=use_cpu_initialization,
+                megatron_amp_O2=megatron_amp_O2,
                 hidden_dropout=decoder_cfg.get('hidden_dropout', 0.1),
                 attention_dropout=decoder_cfg.get('attention_dropout', 0.1),
                 ffn_dropout=decoder_cfg.get('ffn_dropout', 0.0),
@@ -361,6 +401,7 @@ class MegatronTokenLevelEncoderDecoderModule(MegatronModule):
                     gather_output=not self.parallel_output,
                     init_method=init_method_normal(decoder_cfg.init_method_std),
                     use_cpu_initialization=use_cpu_initialization,
+                    params_dtype=self.dtype,
                 )
 
             self._tokens_head_key = 'tokens_head'
