@@ -14,7 +14,10 @@
 
 """Utilities for generating text."""
 
+import pickle
 from collections.abc import Iterable
+from functools import partial
+from typing import Callable, Tuple
 
 import numpy as np
 import torch
@@ -27,13 +30,23 @@ from nemo.collections.nlp.modules.common.transformer.text_generation import Leng
 from nemo.utils import AppState
 
 try:
-    from apex.transformer import parallel_state, tensor_parallel
     from apex.transformer.pipeline_parallel.utils import _reconfigure_microbatch_calculator
 
     HAVE_APEX = True
+
 except (ImportError, ModuleNotFoundError):
+
     HAVE_APEX = False
 
+try:
+    from megatron.core import parallel_state, tensor_parallel
+
+    HAVE_MEGATRON_CORE = True
+
+except (ImportError, ModuleNotFoundError):
+
+    HAVE_MEGATRON_CORE = False
+
 __all__ = [
     "get_default_sampling_params",
     "get_default_length_params",
@@ -122,36 +135,41 @@ def megatron_gpt_generate(model, inputs, tokenizer, length_params, sampling_para
 
 
 def get_computeprob_response(tokenizer, response, inputs):
-    compute_prob_response = {}
-    new_token_ids = []
-    new_tokens = []
-    new_texts = []
-    log_probs = []
-    full_logprobs = []
-    offsets = []
-    for batch_id in range(len(response['tokens'])):
-        if isinstance(inputs, (list, tuple)):
-            if isinstance(inputs[0], str):
-                new_token_id = tokenizer.text_to_ids(inputs[batch_id])
-                new_text = inputs[batch_id]
-                token_len = len(new_token_id)
-            elif isinstance(inputs[0], torch.Tensor):
-                token_len = int(inputs[1][batch_id].item())
-                new_token_id = inputs[0][batch_id][:token_len].tolist()
-                new_text = tokenizer.ids_to_text(new_token_id)
-        new_token_ids.append(new_token_id)
-        new_tokens.append(response['tokens'][batch_id][:token_len])
-        new_texts.append(new_text)
-        log_probs.append(response['logprob'][batch_id][:token_len])
-        full_logprobs.append(response['full_logprob'][batch_id][:token_len])
-        offsets.append(response['offsets'][batch_id][:-1])
-    compute_prob_response['sentences'] = new_texts
-    compute_prob_response['tokens'] = new_tokens
-    compute_prob_response['token_ids'] = new_token_ids
-    compute_prob_response['logprob'] = log_probs
-    compute_prob_response['full_logprob'] = full_logprobs
-    compute_prob_response['offsets'] = offsets
-    return compute_prob_response
+    if parallel_state.is_pipeline_first_stage() or parallel_state.is_pipeline_last_stage():
+        # we only have a response on the first and last pipeline stages
+        compute_prob_response = {}
+        new_token_ids = []
+        new_tokens = []
+        new_texts = []
+        log_probs = []
+        full_logprobs = []
+        offsets = []
+        for batch_id in range(len(response['tokens'])):
+            if isinstance(inputs, (list, tuple)):
+                if isinstance(inputs[0], str):
+                    new_token_id = tokenizer.text_to_ids(inputs[batch_id])
+                    new_text = inputs[batch_id]
+                    token_len = len(new_token_id)
+                elif isinstance(inputs[0], torch.Tensor):
+                    token_len = int(inputs[1][batch_id].item())
+                    new_token_id = inputs[0][batch_id][:token_len].tolist()
+                    new_text = tokenizer.ids_to_text(new_token_id)
+            new_token_ids.append(new_token_id)
+            new_tokens.append(response['tokens'][batch_id][:token_len])
+            new_texts.append(new_text)
+            log_probs.append(response['logprob'][batch_id][:token_len])
+            full_logprobs.append(response['full_logprob'][batch_id][:token_len])
+            offsets.append(response['offsets'][batch_id][:-1])
+        compute_prob_response['sentences'] = new_texts
+        compute_prob_response['tokens'] = new_tokens
+        compute_prob_response['token_ids'] = new_token_ids
+        compute_prob_response['logprob'] = log_probs
+        compute_prob_response['full_logprob'] = full_logprobs
+        compute_prob_response['offsets'] = offsets
+        return compute_prob_response
+    else:
+        # intermediate stages
+        return None
 
 
 def get_batch(model, tokenizer, context_tokens):
@@ -176,17 +194,29 @@ def tab_logits(logits, min_id, max_id, filter_value=-float('Inf')):
     return logits
 
 
-def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf')):
-    """ This function has been mostly taken from huggingface conversational
-     ai code at
+def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf'), started=None):
+    """
+       This function has been mostly taken from huggingface conversational
+         ai code at
          https://medium.com/huggingface/how-to-build-a-state-of-the-art-
-              conversational-ai-with-transfer-learning-2d818ac26313 """
+              conversational-ai-with-transfer-learning-2d818ac26313 
 
+        @param logits: logits tensor
+        @param top_k: keep only top k tokens with highest probability
+        @param top_p: keep the top tokens with cumulative probability
+        @filter_value: value to set filtered tokens to
+        @started: a tensor of bools indicating whether the text generation starts for the batch
+        returns the filtered logits
+    """
     if top_k > 0:
         # Remove all tokens with a probability less than the
         # last token of the top-k
         indices_to_remove = logits < torch.topk(logits, top_k)[0][..., -1, None]
-        logits[indices_to_remove] = filter_value
+        if started is not None:
+            for i in np.arange(indices_to_remove.size(0))[started.cpu().numpy()]:
+                logits[i, indices_to_remove[i]] = filter_value
+        else:
+            logits[indices_to_remove] = filter_value
 
     if top_p > 0.0:
         # Cconvert to 1D
@@ -199,9 +229,14 @@ def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf')):
         # above the threshold
         sorted_indices_to_remove[..., 1:] = sorted_indices_to_remove[..., :-1].clone()
         sorted_indices_to_remove[..., 0] = 0
-        for i in range(sorted_indices.size(0)):
-            indices_to_remove = sorted_indices[i][sorted_indices_to_remove[i]]
-            logits[i][indices_to_remove] = filter_value
+        if started is not None:
+            for i in np.arange(sorted_indices.size(0))[started.cpu().numpy()]:
+                indices_to_remove = sorted_indices[i][sorted_indices_to_remove[i]]
+                logits[i, indices_to_remove] = filter_value
+        else:
+            for i in range(sorted_indices.size(0)):
+                indices_to_remove = sorted_indices[i][sorted_indices_to_remove[i]]
+                logits[i, indices_to_remove] = filter_value
 
     return logits
 
@@ -240,6 +275,7 @@ def send_generate_info(
     greedy,
     repetition_penalty,
     min_tokens_to_generate,
+    end_strings,
 ):
     """
     Needs to be synced up with receive_generate_info
@@ -266,6 +302,14 @@ def send_generate_info(
     torch.distributed.broadcast(context_length_tensor, src, model_parallel_group)
     torch.distributed.broadcast(context_tokens_tensor, src, model_parallel_group)
 
+    # send end strings
+    string_tensor = torch.as_tensor(
+        np.frombuffer(pickle.dumps(end_strings), dtype=np.int8), device=torch.cuda.current_device()
+    )
+    size = torch.as_tensor([string_tensor.size(0)], device=torch.cuda.current_device(), dtype=torch.int64)
+    torch.distributed.broadcast(size, src, model_parallel_group)
+    torch.distributed.broadcast(string_tensor, src, model_parallel_group)
+
 
 def receive_generate_info():
     """
@@ -292,6 +336,14 @@ def receive_generate_info():
     torch.distributed.broadcast(context_length_tensor, src, model_parallel_group)
     torch.distributed.broadcast(context_tokens_tensor, src, model_parallel_group)
 
+    array_size = torch.empty(1, dtype=torch.int64, device=torch.cuda.current_device())
+    torch.distributed.broadcast(array_size, src, model_parallel_group)
+
+    string_tensor = torch.empty(array_size[0], dtype=torch.int8, device=torch.cuda.current_device())
+    torch.distributed.broadcast(string_tensor, src, model_parallel_group)
+    bytes = string_tensor.cpu().numpy().tobytes()
+    end_strings = pickle.loads(bytes)
+
     return (
         context_length_tensor,
         context_tokens_tensor,
@@ -303,6 +355,7 @@ def receive_generate_info():
         greedy,
         repetition_penalty,
         min_tokens_to_generate,
+        end_strings,
     )
 
 
@@ -319,6 +372,7 @@ def synced_generate(
     greedy=False,
     repetition_penalty=1.2,
     min_tokens_to_generate=0,
+    end_strings=[],
 ):
     context_length = context_length_tensor.min().item()
     tokenizer = model.tokenizer
@@ -341,6 +395,7 @@ def synced_generate(
             tokens_to_generate,
             all_probs,
             temperature=temperature,
+            end_strings=end_strings,
             extra={
                 "top_p": top_p,
                 "top_k": top_k,
@@ -398,6 +453,7 @@ def generate(
     greedy=False,
     repetition_penalty=1.0,
     min_tokens_to_generate=0,
+    end_strings=['<|endoftext|>'],
     **strategy_args,
 ) -> OutputType:
     """
@@ -414,6 +470,7 @@ def generate(
         repetition_penalty (float): The parameter for repetition penalty. 1.0 means no penalty
         min_tokens_to_generate (int): The minimum length of the tokens to be generated
         strategy_args, the extra arguments are treated as inference strategy arguments
+        end_strings, a list of strings to stop generation when they are encountered in the output.
     Returns:
         OutputType: It generates the output in a dictionary type. It has the following keys:
             sentences: List[str], output sentences
@@ -447,6 +504,7 @@ def generate(
             greedy,
             repetition_penalty,
             min_tokens_to_generate,
+            end_strings,
         )
     else:
         (
@@ -460,6 +518,7 @@ def generate(
             greedy,
             repetition_penalty,
             min_tokens_to_generate,
+            end_strings,
         ) = receive_generate_info()
 
     output = synced_generate(
@@ -475,6 +534,7 @@ def generate(
         greedy=greedy,
         repetition_penalty=repetition_penalty,
         min_tokens_to_generate=min_tokens_to_generate,
+        end_strings=end_strings,
     )
     special_tokens = set()
     if hasattr(tokenizer, 'pad_token') and tokenizer.pad_token is not None:
@@ -555,6 +615,7 @@ def sample_sequence_batch(
     all_probs=False,
     type_ids=None,
     temperature=None,
+    end_strings=['<|endoftext|>'],
     extra={},
 ):
     # Importing here to avoid circular import errors
@@ -621,6 +682,8 @@ def sample_sequence_batch(
                 # make sure it won't sample outside the vocab_size range
                 logits[:, tokenizer.vocab_size :] = -float('Inf')
 
+                # started indicates whether the current token step passes the context_length, so we make sure not to overwrite the context tokens
+                started = context_lengths <= context_length
                 if extra.get('greedy', False):
                     prev = torch.argmax(logits, dim=-1).view(-1)
                 else:
@@ -628,10 +691,11 @@ def sample_sequence_batch(
                     logits /= temperature
                     # handle repetition penality
                     logits = repetition_penalty(logits, extra.get('repetition_penalty', 1.2), all_generated_indices)
-                    logits = top_k_logits(logits, top_k=extra.get('top_k', 0), top_p=extra.get('top_p', 0.9))
-                    log_probs = F.softmax(logits, dim=-1)
-                    prev = torch.multinomial(log_probs, num_samples=1).view(-1)
-                started = context_lengths <= context_length
+                    logits = top_k_logits(
+                        logits, top_k=extra.get('top_k', 0), top_p=extra.get('top_p', 0.9), started=started
+                    )
+                    probs = F.softmax(logits, dim=-1)
+                    prev = torch.multinomial(probs, num_samples=1).view(-1)
 
                 # Clamp the predicted out of vocabulary tokens
                 prev = torch.clamp(prev, max=tokenizer.vocab_size - 1)
@@ -668,7 +732,12 @@ def sample_sequence_batch(
                 group = parallel_state.get_embedding_group()
                 torch.distributed.broadcast(new_tokens, src, group)
 
-                done_token = (prev == eod_id).byte() & started.byte()
+                #                done_token = (prev == eod_id).byte() & started.byte()
+                done_token = inference_strategy.end_of_generation_condition(
+                    tokens[:, : context_length + 1], prev, eod_id, end_strings
+                )
+                done_token = done_token.byte() & started.byte()
+
                 just_finished = (done_token & ~is_done).bool()
                 lengths[just_finished.view(-1)] = context_length
                 is_done = is_done | done_token
@@ -792,8 +861,8 @@ def tab_sample_sequence_batch(
                     # limit the range
                     min_id, max_id = tokenid_range[token_in_row]
                     logits = tab_logits(logits, min_id, max_id)
-                log_probs = F.softmax(logits, dim=-1)
-                prev = torch.multinomial(log_probs, num_samples=1).view(-1)
+                probs = F.softmax(logits, dim=-1)
+                prev = torch.multinomial(probs, num_samples=1).view(-1)
                 started = context_lengths <= context_length
                 # Clamp the out of vocabulary tokens.
                 prev = torch.clamp(prev, max=tokenizer.vocab_size - 1)
@@ -901,3 +970,96 @@ def sample_token_topk(logits, top_k=0, top_p=0.0, temperature=1.0, filter_value=
     log_probs = log_probs.gather(1, token_ids.unsqueeze(1)).squeeze(1)
 
     return log_probs, token_ids
+
+
+def sample_token_topk_beam_search(logits: torch.Tensor, beam_size: int = 1, dim: int = -1, log_softmax: bool = True):
+    """
+    Beam search selection of top K predictions per target (dim). Returns the beam_size tokens ids with the highest
+    probability and the corresponding log_prob per target
+
+    Args:
+        logits: [batch_size, vocab_size] or [batch_size, vocab_size] - unnormalized log probabilities of the next token,
+        beam_size: int > 1 - number of tokens to return with the highest probability per target
+        dim: int - dim of log_softmax and topk selection
+        log_softmax: bool - if to calculate log softmax  for log probabilities
+
+
+    Returns:
+        log_probs: [batch_size, beam_size] - log probabilities of the sampled tokens
+        token_ids: [batch_size, beam_size] - sampled token ids
+    """
+    if log_softmax:
+        log_probs = torch.nn.functional.log_softmax(logits, dim=dim)
+    else:
+        log_probs = logits
+    # get top candidates for each item in batch
+    log_probs, token_ids = torch.topk(log_probs, beam_size, dim=dim)
+
+    return log_probs, token_ids
+
+
+def compute_beam_search_len_penalty(lengths: torch.Tensor, alpha: int) -> torch.Tensor:
+    """
+    Length penalty used in the beam search
+    Args:
+        lengths: lengths of decoded sequences
+        alpha: params of the penalty
+    Returns:
+         tensor with the penalty value
+    """
+    return ((5 + lengths) / 6).pow(alpha)
+
+
+def get_sampling_token_fn(sampling_method: str, sampling_kwargs: dict) -> Tuple[Callable, dict]:
+    """
+    Specifies the sampling function that takes in a tensor of logits [batch_size, vocab_size] and returns a tuple
+    (tensor of log_probs [batch_size], tensor of sampled from logits [batch_size]).
+    If the beam search is enabled, the sampling function returns tensors [batch_size, beam_size]
+
+    Args:
+        sampling_method: the sampling method to use in the decode steps. Currently supported methods are
+                          "beam-search"/"greedy"/"topkp"
+        sampling_kwargs: dict with arguments to be passed to the sampling function.
+                         For sampling method 'beam-search', the following kwargs are supported:
+                         beam_size - int, number of the best sequences at each decode iteration to be left per target
+                         beam_alpha - int, the parameter of length penalty applied to predicted sequences
+                         keep_only_best_tokens - used in the beam search, boolean flag if to output only best sequence
+                                                 of predicted tokens (True) or beam_size predictions per target
+                         return_scores - used in the beam search, boolean flag if to return scores at the top of
+                                         predictions and logits
+
+    Returns:
+        sample_token_fn: the sampling function
+        default_sampling_kwargs: sampling_kwargs augmented with default sampling kwargs
+    """
+    all_default_sampling_kwargs = {
+        'greedy-search': {},
+        'topkp-sampling': {'top_k': 0, 'top_p': 0.0, 'temperature': 1.0},
+        'beam-search': {'beam_size': 1, 'beam_alpha': 0.0, 'keep_only_best_tokens': False, 'return_scores': False},
+    }
+
+    # update default sampling kwargs with user provided kwargs
+    default_sampling_kwargs = all_default_sampling_kwargs[sampling_method].copy()
+    default_sampling_kwargs.update(sampling_kwargs)
+    # sampling_kwargs = default_sampling_kwargs
+
+    if sampling_method == 'greedy-search':
+        sampling_token_fn = sample_token_greedy
+
+    elif sampling_method == "topkp-sampling":
+        top_k = default_sampling_kwargs['top_k']
+        top_p = default_sampling_kwargs['top_p']
+        temperature = default_sampling_kwargs['temperature']
+        sampling_token_fn = partial(sample_token_topk, top_k=top_k, top_p=top_p, temperature=temperature)
+
+    elif sampling_method == "beam-search":
+        beam_size = default_sampling_kwargs['beam_size']
+        sampling_token_fn = partial(sample_token_topk_beam_search, beam_size=beam_size)
+
+    else:
+        raise ValueError(
+            f'Invalid sampling method {sampling_method}. '
+            f'Supported sampling methods are {all_default_sampling_kwargs.keys()}'
+        )
+
+    return sampling_token_fn, default_sampling_kwargs
