@@ -130,7 +130,6 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
         attn_prior = batch['align_prior_matrix']
         f0 = batch['pitch']
         voiced_mask = batch['voiced_mask']
-        p_voiced = batch['p_voiced']
         energy_avg = batch['energy']
 
         if (
@@ -154,7 +153,6 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
             f0=f0,
             energy_avg=energy_avg,
             voiced_mask=voiced_mask,
-            p_voiced=p_voiced,
         )
         loss_outputs = self.criterion(outputs, in_lens, out_lens)
 
@@ -185,7 +183,6 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
         attn_prior = batch['align_prior_matrix']
         f0 = batch['pitch']
         voiced_mask = batch['voiced_mask']
-        p_voiced = batch['p_voiced']
         energy_avg = batch['energy']
         mel = batch['log_mel']
         if (
@@ -208,7 +205,6 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
             f0=f0,
             energy_avg=energy_avg,
             voiced_mask=voiced_mask,
-            p_voiced=p_voiced,
         )
         loss_outputs = self.criterion(outputs, in_lens, out_lens)
 
@@ -341,7 +337,7 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
                 cfg.text_tokenizer.g2p['_target_'] = cfg.text_tokenizer.g2p['_target_'].replace(
                     "nemo_text_processing.g2p", "nemo.collections.tts.g2p"
                 )
-                logging.warning("This checkpoint support will be dropped after r1.18.0.")
+                logging.warning("This checkpoint support will be dropped after NeMo 1.18.0.")
 
             g2p_kwargs = {}
 
@@ -509,9 +505,6 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
             speaker_id_text=speaker_id_text,
             speaker_id_attributes=speaker_id_attributes,
             sigma=0.7,
-            sigma_txt=0.7,
-            sigma_f0=1.0,
-            sigma_energy=1.0,
             f0_mean=0.0,
             f0_std=0.0,
             in_lens=lens,
