@@ -44,8 +44,9 @@
 
 import torch
 
+from nemo.collections.tts.modules.submodules import ConditionalInput, ConditionalLayerNorm
 from nemo.collections.tts.parts.utils.helpers import binarize_attention_parallel, regulate_len
-from nemo.core.classes import NeuralModule, typecheck
+from nemo.core.classes import NeuralModule, adapter_mixins, typecheck
 from nemo.core.neural_types.elements import (
     EncodedRepresentation,
     Index,
@@ -79,40 +80,53 @@ def average_features(pitch, durs):
     return pitch_avg
 
 
-class ConvReLUNorm(torch.nn.Module):
-    def __init__(self, in_channels, out_channels, kernel_size=1, dropout=0.0):
+class ConvReLUNorm(torch.nn.Module, adapter_mixins.AdapterModuleMixin):
+    def __init__(self, in_channels, out_channels, kernel_size=1, dropout=0.0, condition_dim=384, condition_types=[]):
         super(ConvReLUNorm, self).__init__()
         self.conv = torch.nn.Conv1d(in_channels, out_channels, kernel_size=kernel_size, padding=(kernel_size // 2))
-        self.norm = torch.nn.LayerNorm(out_channels)
+        self.norm = ConditionalLayerNorm(out_channels, condition_dim=condition_dim, condition_types=condition_types)
         self.dropout = torch.nn.Dropout(dropout)
 
-    def forward(self, signal):
+    def forward(self, signal, conditioning=None):
         out = torch.nn.functional.relu(self.conv(signal))
-        out = self.norm(out.transpose(1, 2)).transpose(1, 2)
-        return self.dropout(out)
+        out = self.norm(out.transpose(1, 2), conditioning).transpose(1, 2)
+        out = self.dropout(out)
+
+        if self.is_adapter_available():
+            out = self.forward_enabled_adapters(out.transpose(1, 2)).transpose(1, 2)
+
+        return out
 
 
 class TemporalPredictor(NeuralModule):
     """Predicts a single float per each temporal location"""
 
-    def __init__(self, input_size, filter_size, kernel_size, dropout, n_layers=2):
+    def __init__(self, input_size, filter_size, kernel_size, dropout, n_layers=2, condition_types=[]):
         super(TemporalPredictor, self).__init__()
-
-        self.layers = torch.nn.Sequential(
-            *[
+        self.cond_input = ConditionalInput(input_size, input_size, condition_types)
+        self.layers = torch.nn.ModuleList()
+        for i in range(n_layers):
+            self.layers.append(
                 ConvReLUNorm(
-                    input_size if i == 0 else filter_size, filter_size, kernel_size=kernel_size, dropout=dropout
+                    input_size if i == 0 else filter_size,
+                    filter_size,
+                    kernel_size=kernel_size,
+                    dropout=dropout,
+                    condition_dim=input_size,
+                    condition_types=condition_types,
                 )
-                for i in range(n_layers)
-            ]
-        )
+            )
         self.fc = torch.nn.Linear(filter_size, 1, bias=True)
 
+        # Use for adapter input dimension
+        self.filter_size = filter_size
+
     @property
     def input_types(self):
         return {
             "enc": NeuralType(('B', 'T', 'D'), EncodedRepresentation()),
             "enc_mask": NeuralType(('B', 'T', 1), TokenDurationType()),
+            "conditioning": NeuralType(('B', 'T', 'D'), EncodedRepresentation(), optional=True),
         }
 
     @property
@@ -121,14 +135,20 @@ class TemporalPredictor(NeuralModule):
             "out": NeuralType(('B', 'T'), EncodedRepresentation()),
         }
 
-    def forward(self, enc, enc_mask):
+    def forward(self, enc, enc_mask, conditioning=None):
+        enc = self.cond_input(enc, conditioning)
         out = enc * enc_mask
-        out = self.layers(out.transpose(1, 2)).transpose(1, 2)
+        out = out.transpose(1, 2)
+
+        for layer in self.layers:
+            out = layer(out, conditioning=conditioning)
+
+        out = out.transpose(1, 2)
         out = self.fc(out) * enc_mask
         return out.squeeze(-1)
 
 
-class FastPitchModule(NeuralModule):
+class FastPitchModule(NeuralModule, adapter_mixins.AdapterModuleMixin):
     def __init__(
         self,
         encoder_module: NeuralModule,
@@ -137,15 +157,14 @@ class FastPitchModule(NeuralModule):
         pitch_predictor: NeuralModule,
         energy_predictor: NeuralModule,
         aligner: NeuralModule,
+        speaker_encoder: NeuralModule,
         n_speakers: int,
         symbols_embedding_dim: int,
         pitch_embedding_kernel_size: int,
         energy_embedding_kernel_size: int,
         n_mel_channels: int = 80,
         max_token_duration: int = 75,
-        speaker_emb_condition_prosody: bool = False,
-        speaker_emb_condition_decoder: bool = False,
-        speaker_emb_condition_aligner: bool = False,
+        use_log_energy: bool = True,
     ):
         super().__init__()
 
@@ -155,14 +174,16 @@ class FastPitchModule(NeuralModule):
         self.pitch_predictor = pitch_predictor
         self.energy_predictor = energy_predictor
         self.aligner = aligner
+        self.speaker_encoder = speaker_encoder
         self.learn_alignment = aligner is not None
         self.use_duration_predictor = True
         self.binarize = False
-        self.speaker_emb_condition_prosody = speaker_emb_condition_prosody
-        self.speaker_emb_condition_decoder = speaker_emb_condition_decoder
-        self.speaker_emb_condition_aligner = speaker_emb_condition_aligner
+        self.use_log_energy = use_log_energy
 
-        if n_speakers > 1:
+        # TODO: combine self.speaker_emb with self.speaker_encoder
+        # cfg: remove `n_speakers`, create `speaker_encoder.lookup_module`
+        # state_dict: move `speaker_emb.weight` to `speaker_encoder.lookup_module.table.weight`
+        if n_speakers > 1 and speaker_encoder is None:
             self.speaker_emb = torch.nn.Embedding(n_speakers, symbols_embedding_dim)
         else:
             self.speaker_emb = None
@@ -204,6 +225,8 @@ class FastPitchModule(NeuralModule):
             "attn_prior": NeuralType(('B', 'T_spec', 'T_text'), ProbsType(), optional=True),
             "mel_lens": NeuralType(('B'), LengthsType(), optional=True),
             "input_lens": NeuralType(('B'), LengthsType(), optional=True),
+            "reference_spec": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType(), optional=True),
+            "reference_spec_lens": NeuralType(('B'), LengthsType(), optional=True),
         }
 
     @property
@@ -223,6 +246,19 @@ class FastPitchModule(NeuralModule):
             "energy_tgt": NeuralType(('B', 'T_audio'), RegressionValuesType()),
         }
 
+    def get_speaker_embedding(self, batch_size, speaker, reference_spec, reference_spec_lens):
+        """spk_emb: Bx1xD"""
+        if self.speaker_encoder is not None:
+            spk_emb = self.speaker_encoder(batch_size, speaker, reference_spec, reference_spec_lens).unsqueeze(1)
+        elif self.speaker_emb is not None:
+            if speaker is None:
+                raise ValueError('Please give speaker id to get lookup speaker embedding.')
+            spk_emb = self.speaker_emb(speaker).unsqueeze(1)
+        else:
+            spk_emb = None
+
+        return spk_emb
+
     @typecheck()
     def forward(
         self,
@@ -237,6 +273,8 @@ class FastPitchModule(NeuralModule):
         attn_prior=None,
         mel_lens=None,
         input_lens=None,
+        reference_spec=None,
+        reference_spec_lens=None,
     ):
 
         if not self.learn_alignment and self.training:
@@ -244,34 +282,31 @@ class FastPitchModule(NeuralModule):
             assert pitch is not None
 
         # Calculate speaker embedding
-        if self.speaker_emb is None or speaker is None:
-            spk_emb = 0
-        else:
-            spk_emb = self.speaker_emb(speaker).unsqueeze(1)
+        spk_emb = self.get_speaker_embedding(
+            batch_size=text.shape[0],
+            speaker=speaker,
+            reference_spec=reference_spec,
+            reference_spec_lens=reference_spec_lens,
+        )
 
         # Input FFT
         enc_out, enc_mask = self.encoder(input=text, conditioning=spk_emb)
-        if self.speaker_emb_condition_prosody:
-            prosody_input = enc_out + spk_emb
-        else:
-            prosody_input = enc_out
-        log_durs_predicted = self.duration_predictor(prosody_input, enc_mask)
+
+        # Predict duration
+        log_durs_predicted = self.duration_predictor(enc_out, enc_mask, conditioning=spk_emb)
         durs_predicted = torch.clamp(torch.exp(log_durs_predicted) - 1, 0, self.max_token_duration)
 
         attn_soft, attn_hard, attn_hard_dur, attn_logprob = None, None, None, None
         if self.learn_alignment and spec is not None:
             text_emb = self.encoder.word_emb(text)
-            if self.speaker_emb_condition_aligner and not isinstance(spk_emb, int):
-                attn_soft, attn_logprob = self.aligner(
-                    spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior, conditioning=spk_emb
-                )
-            else:
-                attn_soft, attn_logprob = self.aligner(spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior)
+            attn_soft, attn_logprob = self.aligner(
+                spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior, conditioning=spk_emb
+            )
             attn_hard = binarize_attention_parallel(attn_soft, input_lens, mel_lens)
             attn_hard_dur = attn_hard.sum(2)[:, 0, :]
 
         # Predict pitch
-        pitch_predicted = self.pitch_predictor(prosody_input, enc_mask)
+        pitch_predicted = self.pitch_predictor(enc_out, enc_mask, conditioning=spk_emb)
         if pitch is not None:
             if self.learn_alignment and pitch.shape[-1] != pitch_predicted.shape[-1]:
                 # Pitch during training is per spectrogram frame, but during inference, it should be per character
@@ -287,7 +322,7 @@ class FastPitchModule(NeuralModule):
 
         # Predict energy
         if self.energy_predictor is not None:
-            energy_pred = self.energy_predictor(prosody_input, enc_mask).squeeze(-1)
+            energy_pred = self.energy_predictor(enc_out, enc_mask, conditioning=spk_emb).squeeze(-1)
 
             if energy is not None:
                 # Average energy over characters
@@ -295,7 +330,8 @@ class FastPitchModule(NeuralModule):
                     energy_tgt = average_features(energy.unsqueeze(1), attn_hard_dur)
                 else:
                     energy_tgt = average_features(energy.unsqueeze(1), durs_predicted)
-                energy_tgt = torch.log(1.0 + energy_tgt)
+                if self.use_log_energy:
+                    energy_tgt = torch.log(1.0 + energy_tgt)
                 energy_emb = self.energy_emb(energy_tgt)
                 energy_tgt = energy_tgt.squeeze(1)
             else:
@@ -314,12 +350,13 @@ class FastPitchModule(NeuralModule):
         # Use predictions during inference
         elif spec is None:
             len_regulated, dec_lens = regulate_len(durs_predicted, enc_out, pace)
+        else:
+            raise ValueError(
+                f"Something unexpected happened when 'spec' is not None and 'self.learn_alignment' is False."
+            )
 
         # Output FFT
-        if self.speaker_emb_condition_decoder:
-            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
-        else:
-            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
         spect = self.proj(dec_out).transpose(1, 2)
         return (
             spect,
@@ -336,26 +373,35 @@ class FastPitchModule(NeuralModule):
             energy_tgt,
         )
 
-    def infer(self, *, text, pitch=None, speaker=None, energy=None, pace=1.0, volume=None):
+    def infer(
+        self,
+        *,
+        text,
+        pitch=None,
+        speaker=None,
+        energy=None,
+        pace=1.0,
+        volume=None,
+        reference_spec=None,
+        reference_spec_lens=None,
+    ):
         # Calculate speaker embedding
-        if self.speaker_emb is None or speaker is None:
-            spk_emb = 0
-        else:
-            spk_emb = self.speaker_emb(speaker).unsqueeze(1)
+        spk_emb = self.get_speaker_embedding(
+            batch_size=text.shape[0],
+            speaker=speaker,
+            reference_spec=reference_spec,
+            reference_spec_lens=reference_spec_lens,
+        )
 
         # Input FFT
         enc_out, enc_mask = self.encoder(input=text, conditioning=spk_emb)
-        if self.speaker_emb_condition_prosody:
-            prosody_input = enc_out + spk_emb
-        else:
-            prosody_input = enc_out
 
         # Predict duration and pitch
-        log_durs_predicted = self.duration_predictor(prosody_input, enc_mask)
+        log_durs_predicted = self.duration_predictor(enc_out, enc_mask, conditioning=spk_emb)
         durs_predicted = torch.clamp(
             torch.exp(log_durs_predicted) - 1.0, self.min_token_duration, self.max_token_duration
         )
-        pitch_predicted = self.pitch_predictor(prosody_input, enc_mask) + pitch
+        pitch_predicted = self.pitch_predictor(enc_out, enc_mask, conditioning=spk_emb) + pitch
         pitch_emb = self.pitch_emb(pitch_predicted.unsqueeze(1))
         enc_out = enc_out + pitch_emb.transpose(1, 2)
 
@@ -364,7 +410,7 @@ class FastPitchModule(NeuralModule):
                 assert energy.shape[-1] == text.shape[-1], f"energy.shape[-1]: {energy.shape[-1]} != len(text)"
                 energy_emb = self.energy_emb(energy)
             else:
-                energy_pred = self.energy_predictor(prosody_input, enc_mask).squeeze(-1)
+                energy_pred = self.energy_predictor(enc_out, enc_mask, conditioning=spk_emb).squeeze(-1)
                 energy_emb = self.energy_emb(energy_pred.unsqueeze(1))
             enc_out = enc_out + energy_emb.transpose(1, 2)
 
@@ -376,10 +422,7 @@ class FastPitchModule(NeuralModule):
             volume_extended = volume_extended.squeeze(-1).float()
 
         # Output FFT
-        if self.speaker_emb_condition_decoder:
-            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
-        else:
-            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
         spect = self.proj(dec_out).transpose(1, 2)
         return (
             spect.to(torch.float),
