@@ -17,9 +17,9 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
-from nemo.collections.tts.modules.submodules import LinearNorm
+from nemo.collections.tts.modules.submodules import ConditionalInput, ConditionalLayerNorm, LinearNorm
 from nemo.collections.tts.parts.utils.helpers import get_mask_from_lengths
-from nemo.core.classes import NeuralModule, typecheck
+from nemo.core.classes import NeuralModule, adapter_mixins, typecheck
 from nemo.core.neural_types.elements import EncodedRepresentation, LengthsType, MaskType, TokenIndex
 from nemo.core.neural_types.neural_type import NeuralType
 
@@ -51,7 +51,7 @@ class PositionalEmbedding(nn.Module):
 
 
 class PositionwiseConvFF(nn.Module):
-    def __init__(self, d_model, d_inner, kernel_size, dropout, pre_lnorm=False):
+    def __init__(self, d_model, d_inner, kernel_size, dropout, pre_lnorm=False, condition_types=[]):
         super(PositionwiseConvFF, self).__init__()
 
         self.d_model = d_model
@@ -68,17 +68,17 @@ class PositionwiseConvFF(nn.Module):
             nn.Conv1d(d_inner, d_model, kernel_size[1], 1, (kernel_size[1] // 2)),
             nn.Dropout(dropout),
         )
-        self.layer_norm = nn.LayerNorm(d_model)
+        self.layer_norm = ConditionalLayerNorm(d_model, condition_dim=d_model, condition_types=condition_types)
         self.pre_lnorm = pre_lnorm
 
-    def forward(self, inp):
-        return self._forward(inp)
+    def forward(self, inp, conditioning=None):
+        return self._forward(inp, conditioning)
 
-    def _forward(self, inp):
+    def _forward(self, inp, conditioning=None):
         if self.pre_lnorm:
             # layer normalization + positionwise feed-forward
             core_out = inp.transpose(1, 2)
-            core_out = self.CoreNet(self.layer_norm(core_out).to(inp.dtype))
+            core_out = self.CoreNet(self.layer_norm(core_out, conditioning).to(inp.dtype))
             core_out = core_out.transpose(1, 2)
 
             # residual connection
@@ -90,13 +90,13 @@ class PositionwiseConvFF(nn.Module):
             core_out = core_out.transpose(1, 2)
 
             # residual connection + layer normalization
-            output = self.layer_norm(inp + core_out).to(inp.dtype)
+            output = self.layer_norm(inp + core_out, conditioning).to(inp.dtype)
 
         return output
 
 
 class MultiHeadAttn(nn.Module):
-    def __init__(self, n_head, d_model, d_head, dropout, dropatt=0.1, pre_lnorm=False):
+    def __init__(self, n_head, d_model, d_head, dropout, dropatt=0.1, pre_lnorm=False, condition_types=[]):
         super(MultiHeadAttn, self).__init__()
 
         self.n_head = n_head
@@ -109,17 +109,17 @@ class MultiHeadAttn(nn.Module):
         self.drop = nn.Dropout(dropout)
         self.dropatt = nn.Dropout(dropatt)
         self.o_net = nn.Linear(n_head * d_head, d_model, bias=False)
-        self.layer_norm = nn.LayerNorm(d_model)
+        self.layer_norm = ConditionalLayerNorm(d_model, condition_dim=d_model, condition_types=condition_types)
 
-    def forward(self, inp, attn_mask=None):
-        return self._forward(inp, attn_mask)
+    def forward(self, inp, attn_mask=None, conditioning=None):
+        return self._forward(inp, attn_mask, conditioning)
 
-    def _forward(self, inp, attn_mask=None):
+    def _forward(self, inp, attn_mask=None, conditioning=None):
         residual = inp
 
         if self.pre_lnorm:
             # layer normalization
-            inp = self.layer_norm(inp)
+            inp = self.layer_norm(inp, conditioning)
 
         n_head, d_head = self.n_head, self.d_head
 
@@ -157,29 +157,47 @@ class MultiHeadAttn(nn.Module):
             output = residual + attn_out
         else:
             # residual connection + layer normalization
-            output = self.layer_norm(residual + attn_out)
+            output = self.layer_norm(residual + attn_out, conditioning)
 
         return output
 
 
-class TransformerLayer(nn.Module):
-    def __init__(self, n_head, d_model, d_head, d_inner, kernel_size, dropout, **kwargs):
+class TransformerLayer(nn.Module, adapter_mixins.AdapterModuleMixin):
+    def __init__(self, n_head, d_model, d_head, d_inner, kernel_size, dropout, condition_types=[], **kwargs):
         super(TransformerLayer, self).__init__()
 
-        self.dec_attn = MultiHeadAttn(n_head, d_model, d_head, dropout, **kwargs)
-        self.pos_ff = PositionwiseConvFF(d_model, d_inner, kernel_size, dropout, pre_lnorm=kwargs.get('pre_lnorm'))
+        self.dec_attn = MultiHeadAttn(n_head, d_model, d_head, dropout, condition_types=condition_types, **kwargs)
+        self.pos_ff = PositionwiseConvFF(
+            d_model, d_inner, kernel_size, dropout, pre_lnorm=kwargs.get('pre_lnorm'), condition_types=condition_types
+        )
 
-    def forward(self, dec_inp, mask=None):
-        output = self.dec_attn(dec_inp, attn_mask=~mask.squeeze(2))
+    def forward(self, dec_inp, mask=None, conditioning=None):
+        output = self.dec_attn(dec_inp, attn_mask=~mask.squeeze(2), conditioning=conditioning)
         output *= mask
-        output = self.pos_ff(output)
+        output = self.pos_ff(output, conditioning)
         output *= mask
+
+        if self.is_adapter_available():
+            output = self.forward_enabled_adapters(output)
+            output *= mask
+
         return output
 
 
 class FFTransformerDecoder(NeuralModule):
     def __init__(
-        self, n_layer, n_head, d_model, d_head, d_inner, kernel_size, dropout, dropatt, dropemb=0.0, pre_lnorm=False
+        self,
+        n_layer,
+        n_head,
+        d_model,
+        d_head,
+        d_inner,
+        kernel_size,
+        dropout,
+        dropatt,
+        dropemb=0.0,
+        pre_lnorm=False,
+        condition_types=[],
     ):
         super(FFTransformerDecoder, self).__init__()
         self.d_model = d_model
@@ -189,11 +207,20 @@ class FFTransformerDecoder(NeuralModule):
         self.pos_emb = PositionalEmbedding(self.d_model)
         self.drop = nn.Dropout(dropemb)
         self.layers = nn.ModuleList()
+        self.cond_input = ConditionalInput(d_model, d_model, condition_types)
 
         for _ in range(n_layer):
             self.layers.append(
                 TransformerLayer(
-                    n_head, d_model, d_head, d_inner, kernel_size, dropout, dropatt=dropatt, pre_lnorm=pre_lnorm
+                    n_head,
+                    d_model,
+                    d_head,
+                    d_inner,
+                    kernel_size,
+                    dropout,
+                    dropatt=dropatt,
+                    pre_lnorm=pre_lnorm,
+                    condition_types=condition_types,
                 )
             )
 
@@ -213,16 +240,18 @@ class FFTransformerDecoder(NeuralModule):
         }
 
     @typecheck()
-    def forward(self, input, seq_lens, conditioning=0):
+    def forward(self, input, seq_lens, conditioning=None):
         return self._forward(input, mask_from_lens(seq_lens).unsqueeze(2), conditioning)
 
     def _forward(self, inp, mask, conditioning):
         pos_seq = torch.arange(inp.size(1), device=inp.device).to(inp.dtype)
         pos_emb = self.pos_emb(pos_seq) * mask
-        out = self.drop(inp + pos_emb + conditioning)
+        inp += pos_emb
+        inp = self.cond_input(inp, conditioning)
+        out = self.drop(inp)
 
         for layer in self.layers:
-            out = layer(out, mask=mask)
+            out = layer(out, mask=mask, conditioning=conditioning)
 
         # out = self.drop(out)
         return out, mask
@@ -244,9 +273,20 @@ class FFTransformerEncoder(FFTransformerDecoder):
         n_embed=None,
         d_embed=None,
         padding_idx=0,
+        condition_types=[],
     ):
         super(FFTransformerEncoder, self).__init__(
-            n_layer, n_head, d_model, d_head, d_inner, kernel_size, dropout, dropatt, dropemb, pre_lnorm
+            n_layer,
+            n_head,
+            d_model,
+            d_head,
+            d_inner,
+            kernel_size,
+            dropout,
+            dropatt,
+            dropemb,
+            pre_lnorm,
+            condition_types,
         )
 
         self.padding_idx = padding_idx
