@@ -12,6 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# TODO @xueyang: deprecate this file since no other places import modules from here anymore. However,
-#  all checkpoints uploaded in ngc used this path. So it requires to update all ngc checkpoints g2p path as well.
-from nemo.collections.tts.g2p.modules import IPAG2P, BaseG2p, EnglishG2p
+# TODO @xueyang: This file is kept for backward-compatibility purposes since all older NGC models that were trained on
+#  and before NeMo 1.16.0 used this import path. We will remove this file soon; `IPAG2P` will be also renamed as
+#  `IpaG2p`. Please start using new import path and the new `IpaG2p` name from NeMo 1.16.0.
+from nemo.collections.tts.g2p.models.en_us_arpabet import EnglishG2p
+from nemo.collections.tts.g2p.models.i18n_ipa import IpaG2p as IPAG2P
+from nemo.collections.tts.g2p.models.zh_cn_pinyin import ChineseG2p
